﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;

namespace AirFusion
{
	/// <summary>
	/// Executes the command for delegate functions
	/// </summary>
	public class DelegateCommand : ICommand
	{
		#region Members

		// This action is for executed function when "Execute" stage
		private Action<object> _execute = null;

		// This action is for executed function when "CanExecute" stage
		private Func<object, bool> _canExecute = null;

		#endregion Members

		#region Events

		public event EventHandler CanExecuteChanged;

		#endregion Events

		#region Constructors & Dispose

		public DelegateCommand(Action<object> exec, Func<object, bool> canExec)
		{
			this._execute = exec;
			this._canExecute = canExec;
		}

		#endregion Constructors & Dispose

		#region Public Methods

		public bool CanExecute(object param)
		{
			if (this._canExecute != null)
				return this._canExecute(param);
			return true;
		}

		public void Execute(object param)
		{
			if (this._execute != null)
				this._execute(param);
		}

		public void RaiseCanExecuteChanged()
		{
			if (this.CanExecuteChanged != null)
				this.CanExecuteChanged(this, EventArgs.Empty);
		}

		#endregion Public Methods
	}
}