﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Reflection;
using System.Diagnostics;
using System.Runtime.InteropServices;
using Microsoft.Win32;
using System.Security.AccessControl;
using System.Security.Principal;


namespace AirFusion
{
    public class SingleInstance : IDisposable
	{
		#region Members

		private Mutex _singleInstMutex = null;
        private bool _created = false;

		#endregion

		#region Properties

		public bool IsSingleInstance
        {
            get 
			{ 
				return _created; 
			}
        }

		#endregion

		#region Constructors & Dispose

		public SingleInstance(string id)
        {
            _singleInstMutex = new Mutex(true,id,out _created);
            var allowEveryoneRule = new MutexAccessRule(new SecurityIdentifier(WellKnownSidType.WorldSid, null), MutexRights.FullControl, AccessControlType.Allow);
            var securitySettings = new MutexSecurity();
            securitySettings.AddAccessRule(allowEveryoneRule);
            _singleInstMutex.SetAccessControl(securitySettings);
        }

        ~SingleInstance()
        {
            ReleaseMutex();
        }

		#endregion

		#region Private Methods

		private void ReleaseMutex()
        {
            try
            {
                if (_created)
                {
                    _singleInstMutex.ReleaseMutex();
                    _created = false;
                }
            }
            catch
            {

            }
        }

		#endregion

		#region Public Methods

		public void Dispose()
		{
			// Release mutex
			ReleaseMutex();
			// Notify the garbage collector to ignore the destructor
			GC.SuppressFinalize(this);
		}

		#endregion
	}
}
