﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.Windows;
using AirFusion.Common;

namespace AirFusion
{
	public class VNCController
	{
		#region Members

		private string _serverIP = string.Empty;
		private uint _serverPort = 0;
		private string _serverPassword = string.Empty;
		private string _downloadFileFolder = string.Empty;

		#endregion

		#region Properties

		private VNC.VNCClient _vncclient = null;
		public VNC.VNCClient VNCClient 
		{
			get 
			{
				return _vncclient;
			}
		}

		private bool _isInitializationSuccess = false;
		public bool IsInitializationSuccess 
		{
			get 
			{
				return _isInitializationSuccess;
			}
		}

		private bool _isRunSuccess = false;
		public bool IsRunSuccess
		{
			get
			{
				return _isRunSuccess;
			}
		}

		#endregion

		#region Constructors & Dispose

		public VNCController(string serverIP, uint serverPort, string serverPassword, string downloadFileFolder = null) 
		{
			// Check server IP
			if (!string.IsNullOrEmpty(serverIP))
			{
				this._serverIP = serverIP;
			}
			else 
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "serverIP is incorret", "");
				throw new System.ArgumentException("Parameter cannot be null or empty", "serverIP");
			}

			// Check server port for VNC use
			if (serverPort != 0)
			{
				this._serverPort = serverPort;
			}
			else
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "serverPort is incorret", "");
				throw new System.ArgumentException("Parameter cannot be zero", "serverPort");
			}

			// Check server password for VNC use
			if (!string.IsNullOrEmpty(serverPassword))
			{
				this._serverPassword = serverPassword;
			}
			else 
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "serverPassword is incorret", "");
				throw new System.ArgumentException("Parameter cannot be null or empty", "serverPassword");
			}
			
			_vncclient = new VNC.VNCClient();

			if (null != downloadFileFolder)
			{
				_downloadFileFolder = downloadFileFolder;

				try
				{
					if (!System.IO.Directory.Exists(_downloadFileFolder))
					{
						System.IO.Directory.CreateDirectory(_downloadFileFolder);
					}
				}
				catch(Exception ex)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
				}
			}
		}

		#endregion

		#region Public Methods

		public InitialStatus InitializeVNC()
		{
			LogtoFile.WriteLog("", Configs.DebugLevelValue, "Wait for _vncclient.InitialVNC()", "");

			if (_vncclient.InitialVNC())
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "_vncclient.InitialVNC(), response true", "");

				if (_downloadFileFolder != string.Empty)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "to set download folder", "");
					_vncclient.SetVNCDownloadFolderPath(_downloadFileFolder);
				}

				LogtoFile.WriteLog("", Configs.DebugLevelValue, "Wait for _vncclient.RunVNC()", "");

				try
				{
					VNC.VNC_ERR_CODE errorCode = _vncclient.RunVNC(this._serverIP, this._serverPort, this._serverPassword);
					if (errorCode == VNC.VNC_ERR_CODE.EC_NoError)
					{
						_isInitializationSuccess = true;
						_isRunSuccess = true;
						return InitialStatus.ConnectToInputControlCompleted;
					}
					else
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "_vncclient.RunVNC Error: " + errorCode.ToString(), "");
						_isInitializationSuccess = false;
						_isRunSuccess = false;

						try
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "Wait for _vncclient.CloseVNC()", "");
							_vncclient.CloseVNC();
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "Finish _vncclient.CloseVNC()", "");
						}
						catch (Exception ex)
						{
							LogtoFile.WriteLog("", Configs.DebugLevelValue, "_vncclient.CloseVNC Error: " + ex.ToString(), "");
						}

						return InitialStatus.RunInputControlFailed;
					}
				}
				catch(Exception ex)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
					_isInitializationSuccess = false;
					_isRunSuccess = false;

					try
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Wait for _vncclient.CloseVNC()", "");
						_vncclient.CloseVNC();
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "Finish _vncclient.CloseVNC()", "");
					}
					catch (Exception exi)
					{
						LogtoFile.WriteLog("", Configs.DebugLevelValue, "_vncclient.CloseVNC Error: " + exi.ToString(), "");
					}

					return InitialStatus.RunInputControlFailed;
				}
			}
			else
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, "_vncclient.InitialVNC(), response false", "");
				_isInitializationSuccess = false;
				_isRunSuccess = false;

				try
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Wait for _vncclient.CloseVNC()", "");
					_vncclient.CloseVNC();
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "Finish _vncclient.CloseVNC()", "");
				}
				catch (Exception exi)
				{
					LogtoFile.WriteLog("", Configs.DebugLevelValue, "_vncclient.CloseVNC Error: " + exi.ToString(), "");
				}

				return InitialStatus.InitializeInputControlFailed;
			}
		}

		#endregion
	}
}
