﻿using System.Windows;

namespace AirFusion.Windows
{
	/// <summary>
	/// Interaction logic for CustomMessageBox.xaml
	/// </summary>
	public partial class CustomMessageBox : Window
	{
		#region define Button Style

		public enum CustomImageIcon
		{
			Done,
			Question,
			Suggess,
			Update,
			Warring
		}

		#endregion define Button Style

		public struct CustomMessageBoxStructInfo
		{
			public string MessageInfo, Button1Content, Button2Content, Button3Content;
			public CustomImageIcon imageIcon;
			//public CustomButtonNumber buttonNumber;

			public CustomMessageBoxStructInfo(string messageinfo, CustomImageIcon icon)
			{
				this.MessageInfo = messageinfo;
				this.imageIcon = icon;
				this.Button1Content = Application.Current.TryFindResource("txtOK") as string;
				this.Button2Content = string.Empty;
				this.Button3Content = string.Empty;
			}

			public CustomMessageBoxStructInfo(string messageinfo, CustomImageIcon icon, string button1content)
			{
				this.MessageInfo = messageinfo;
				this.imageIcon = icon;
				this.Button1Content = button1content;
				this.Button2Content = string.Empty;
				this.Button3Content = string.Empty;
			}

			public CustomMessageBoxStructInfo(string messageinfo, CustomImageIcon icon, string button1content, string button2content)
			{
				this.MessageInfo = messageinfo;
				this.imageIcon = icon;
				this.Button1Content = button1content;
				this.Button2Content = button2content;
				this.Button3Content = string.Empty;
			}

			public CustomMessageBoxStructInfo(string messageinfo, CustomImageIcon icon, string button1content, string button2content, string button3content)
			{
				this.MessageInfo = messageinfo;
				this.imageIcon = icon;
				this.Button1Content = button1content;
				this.Button2Content = button2content;
				this.Button3Content = button3content;
			}
		}

		public MessageBoxResult MessageBoxResult { get; set; }

		//public CustomMessageBox(string strMessageInfo, CustomImageIcon imageIcon = CustomImageIcon.Warring, CustomButtonNumber buttonNumber = CustomButtonNumber.Single)
		public CustomMessageBox(CustomMessageBoxStructInfo messageBoxInfo, bool isEnableHyperlink = false, bool isChangeTitle = false)
		{
			InitializeComponent();

			//for AR special Design
			if (System.Globalization.CultureInfo.CurrentUICulture.Parent.Name.ToUpper().Equals("AR"))
				this.FlowDirection = FlowDirection.RightToLeft;

			if(isChangeTitle == false)
				tb_MessageTitle.Text = (string)Application.Current.TryFindResource("txtAppName");
			else
				tb_MessageTitle.Text = (string)Application.Current.TryFindResource("txtUpdateTiltle");

			//Handle Image Icon
			if (messageBoxInfo.imageIcon == CustomImageIcon.Done)
			{
				img_MessageIcon.Style = (Style)Application.Current.TryFindResource(@"CustomerMessageIcon_done_style");
			}
			else if (messageBoxInfo.imageIcon == CustomImageIcon.Question)
			{
				img_MessageIcon.Style = (Style)Application.Current.TryFindResource(@"CustomerMessageIcon_question_style");
			}
			else if (messageBoxInfo.imageIcon == CustomImageIcon.Suggess)
			{
				img_MessageIcon.Style = (Style)Application.Current.TryFindResource(@"CustomerMessageIcon_suggess_style");
			}
			else if (messageBoxInfo.imageIcon == CustomImageIcon.Update)
			{
				img_MessageIcon.Style = (Style)Application.Current.TryFindResource(@"CustomerMessageIcon_update_style");
			}
			else
			{
				img_MessageIcon.Style = (Style)Application.Current.TryFindResource(@"CustomerMessageIcon_warring_style");
			}

			//Handle Button Number
			btn_CustomMessagebtn1.Content = messageBoxInfo.Button1Content;

			//if (buttonNumber == CustomButtonNumber.Double)
			if (!string.IsNullOrEmpty(messageBoxInfo.Button2Content))
			{
				btn_CustomMessagebtn2.Visibility = System.Windows.Visibility.Visible;
				btn_CustomMessagebtn2.Content = messageBoxInfo.Button2Content;
			}
			else if (!string.IsNullOrEmpty(messageBoxInfo.Button2Content) && !string.IsNullOrEmpty(messageBoxInfo.Button3Content))
			{
				btn_CustomMessagebtn3.Visibility = System.Windows.Visibility.Visible;
				btn_CustomMessagebtn3.Content = messageBoxInfo.Button3Content;
			}

			tb_MessageInfo.Text = messageBoxInfo.MessageInfo;

			if (isEnableHyperlink)
			{				
				tb_Hyperlink.Visibility = System.Windows.Visibility.Visible;
			}
			else
			{
				tb_Hyperlink.Visibility = System.Windows.Visibility.Collapsed;
			}
		}

		private void btn_OK_Click(object sender, RoutedEventArgs e)
		{
			this.MessageBoxResult = MessageBoxResult.OK;
			this.Close();
		}

		private void btn_Cancel_Click(object sender, RoutedEventArgs e)
		{
			this.MessageBoxResult = MessageBoxResult.Cancel;
			this.Close();
		}

		private void btn_Close_Click(object sender, RoutedEventArgs e)
		{
			this.MessageBoxResult = MessageBoxResult.None;
			this.Close();
		}

		/// <summary>
		/// Check AcerEXTEND version on mobile device when user taps "OK, I've updated my mobile device".
		/// If user did update AcerEXTEND then go to casting window.
		/// If user didn't update AcerEXTEND then stay on this page.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Hyperlink_Click(object sender, RoutedEventArgs e)
		{
			Configs.ApplicationReference.AppScreenWindow.ConnectNewScreencast(Configs.ApplicationReference.ConnectingDeviceInfo);
			this.Close();
		}
	}
}