﻿using AirFusion.Manager.DeviceManager;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace AirFusion
{
	/// <summary>
	/// Interaction logic for DeviceSettingsPage.xaml
	/// </summary>
	public partial class DeviceSettingsPage : UserControl, INotifyPropertyChanged
	{
		#region Events

		public event EventHandler<UserControlEventArgs> UserControlEvent;

		public event System.ComponentModel.PropertyChangedEventHandler PropertyChanged;

		#endregion Events

		#region Properties

		private AppDeviceInfo _targetDeviceInfo = null;

		public AppDeviceInfo TargetDeviceInfo
		{
			get
			{
				return _targetDeviceInfo;
			}

			set
			{
				if (_targetDeviceInfo != value)
				{
					_targetDeviceInfo = value;
					this.OnPropertyChanged("TargetDeviceInfo");
				}
			}
		}

		private string _currentUserAccount = string.Empty;

		public string CurrentUserAccount
		{
			get
			{
				return _currentUserAccount;
			}

			set
			{
				if (_currentUserAccount != value)
				{
					_currentUserAccount = value;
					this.OnPropertyChanged("CurrentUserAccount");
				}
			}
		}

		#endregion Properties

		#region Constructor & Dispose

		public DeviceSettingsPage()
		{
			InitializeComponent();

			this.DataContext = this;
		}

		#endregion Constructor & Dispose

		#region Protected Methods

		protected virtual void OnPropertyChanged(string propertyName)
		{
			if (this.PropertyChanged != null)
			{
				this.PropertyChanged(this, new System.ComponentModel.PropertyChangedEventArgs(propertyName));
			}
		}

		#endregion Protected Methods

		#region UI Event Callbacks

		private void UserControl_IsVisibleChanged(object sender, DependencyPropertyChangedEventArgs e)
		{
			if (this.Visibility != System.Windows.Visibility.Visible)
			{
				if (null != this.TargetDeviceInfo)
				{
					this.TargetDeviceInfo = null;
					this.CurrentUserAccount = string.Empty;
				}
			}
		}

		private void btnPageBack_Click(object sender, RoutedEventArgs e)
		{
			this.TargetDeviceInfo = null;
			this.CurrentUserAccount = string.Empty;

			if (null != this.UserControlEvent)
			{
				this.UserControlEvent(this, new UserControlEventArgs(UserControlEventType.PageBack, PageType.DeviceSettings));
			}
		}

		private void btnAccountPWDChange_Click(object sender, RoutedEventArgs e)
		{
			// To show change password page
			if (null != this.UserControlEvent)
			{
				this.UserControlEvent(this, new UserControlEventArgs(UserControlEventType.ChangePWD, this.TargetDeviceInfo, null));
			}
		}

		private void tglbtnSendNotification_Checked(object sender, RoutedEventArgs e)
		{
			ToggleButton target = sender as ToggleButton;

			if (null != this.TargetDeviceInfo)
			{
				this.TargetDeviceInfo.IsSendNotificationOn = true;
			}
		}

		private void tglbtnSendNotification_Unchecked(object sender, RoutedEventArgs e)
		{
			ToggleButton target = sender as ToggleButton;

			if (null != this.TargetDeviceInfo)
			{
				this.TargetDeviceInfo.IsSendNotificationOn = false;
			}
		}

		private void ckbAutoUnlock_Checked(object sender, RoutedEventArgs e)
		{
			CheckBox target = sender as CheckBox;

			try
			{
				if (this.Visibility == System.Windows.Visibility.Visible)
				{
					// To show set password page
					if (null != this.UserControlEvent)
					{
						this.UserControlEvent(this, new UserControlEventArgs(UserControlEventType.SetPWD, this.TargetDeviceInfo, null));
					}
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		private void ckbAutoUnlock_Unchecked(object sender, RoutedEventArgs e)
		{
			CheckBox target = sender as CheckBox;

			try
			{
				if (null != this.TargetDeviceInfo)
				{
					// 2015-05-20 Seoker: remove Auto-login & incoming call functions since it's defeatured
					/*
					this.TargetDeviceInfo.IsAutoUnlockOn = false;
					this.TargetDeviceInfo.LoginAccount = string.Empty;
					this.TargetDeviceInfo.LoginPassword = string.Empty;
					*/
					if (null != this.UserControlEvent)
					{
						this.UserControlEvent(this, new UserControlEventArgs(UserControlEventType.DeletePWD, this.TargetDeviceInfo));
					}
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		private void btnDeviceUnpair_Click(object sender, RoutedEventArgs e)
		{
			if (null != this.UserControlEvent)
			{
				this.UserControlEvent(this, new UserControlEventArgs(UserControlEventType.UnpairRequest, this.TargetDeviceInfo));
			}
		}

		#endregion UI Event Callbacks
	}
}
