﻿using AirFusion.Manager.DeviceManager;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace AirFusion
{
	/// <summary>
	/// Interaction logic for ScreenWindowWarringDlg.xaml
	/// </summary>
	public partial class ScreenWindowWarringDlg : UserControl
	{
		#region Events

		public event EventHandler<UserControlEventArgs> UserControlEvent;

		#endregion Events

		#region Properties

		private AppDeviceInfo _targetDeviceInfo = null;

		public AppDeviceInfo TargetDeviceInfo
		{
			get
			{
				return _targetDeviceInfo;
			}

			set
			{
				_targetDeviceInfo = value;
			}
		}

		private UserControlEventType _commandType = UserControlEventType.None;

		private System.Timers.Timer _pTimer = null;// _pTimer = new System.Timers.Timer();
		private const int MONITOR_TIME = 1000; // 1s
		private int totalTime = 60;
		private string strRemaind = string.Empty;

		public UserControlEventType CommandType
		{
			get
			{
				return _commandType;
			}

			set
			{
				_commandType = value;
			}
		}

		#endregion Properties

		public ScreenWindowWarringDlg()
		{
			InitializeComponent();
		}

		private void Window_IsVisibleChanged(object sender, DependencyPropertyChangedEventArgs e)
		{
			try
			{
				if (this.Visibility == System.Windows.Visibility.Visible)
				{
					if (string.IsNullOrEmpty(this.btnLeft.Content as string))
						this.btnLeft.Visibility = System.Windows.Visibility.Collapsed;					
					else
						this.btnLeft.Visibility = System.Windows.Visibility.Visible;

					if (string.IsNullOrEmpty(this.btnRight.Content as string))
						this.btnRight.Visibility = System.Windows.Visibility.Collapsed;
					else
						this.btnRight.Visibility = System.Windows.Visibility.Visible;

					if (string.IsNullOrEmpty(this.btnDebug.Content as string))
						this.btnDebug.Visibility = System.Windows.Visibility.Collapsed;
					else
						this.btnDebug.Visibility = System.Windows.Visibility.Visible;

					//if (string.IsNullOrEmpty(this.b.Content as string))
					//	this.btnRight.Visibility = System.Windows.Visibility.Collapsed;
					//else
					//	this.btnRight.Visibility = System.Windows.Visibility.Visible;

					if (this._commandType == UserControlEventType.StopScreenCast)
					{
						strRemaind = this.txtDialogDetail.Text;

						this.txtDialogDetail.Text = string.Format(strRemaind, totalTime.ToString());												

						//0514 Johnson add for avoid Timer not restart
						if (_pTimer != null)
						{
							_pTimer.Stop();
							_pTimer = null;
						}

						if (_pTimer == null)
						{
							_pTimer = new System.Timers.Timer();
							_pTimer.Interval = MONITOR_TIME;
							_pTimer.Elapsed += RemaindTime;
							_pTimer.Start();
						}
					}
				}
				else
				{
					this.txtRemainTimeDetail.Visibility = Visibility.Collapsed;
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		#region Private Methods

		public void RemaindTime(object sender, EventArgs e)
		{
			this.Dispatcher.Invoke(delegate()
			{
				totalTime--;

				this.txtDialogDetail.Text = string.Format(strRemaind, totalTime.ToString());

				//this.txtRemainTimeDetail.Text = strRemaind + " " + totalTime.ToString() + "s";
			});

			if (totalTime == 0)
			{
				//0514 Johnon modify it, need to send different event to Hint User the screen has been close by AirFusion
				this.UserControlEvent(this, new UserControlEventArgs(UserControlEventType.StopScreenHint, _targetDeviceInfo, true));
				ResetMessage();
			}
		}

		private void ResetMessage()
		{
			this.Dispatcher.Invoke(delegate()
			{
				this.CommandType = UserControlEventType.None;
				this.TargetDeviceInfo = null;
				this.txtDialogTitle.Text = string.Empty;
				this.txtDialogDetail.Text = string.Empty;
				this.imgPresent.Source = null;
				this.txtRemainTimeDetail.Text = string.Empty;
				this.strRemaind = string.Empty;
				if (_pTimer != null)
					_pTimer.Stop();
				_pTimer = null;
				this.totalTime = 60;
			});
		}

		#endregion Private Methods

		#region UI Event Callbacks

		private void btnLeft_Click(object sender, RoutedEventArgs e)
		{
			if (null != this.UserControlEvent)
			{
				this.UserControlEvent(this, new UserControlEventArgs(this.CommandType, _targetDeviceInfo, true));
			}

			this.ResetMessage();
		}

		private void btnRight_Click(object sender, RoutedEventArgs e)
		{
			if (null != this.UserControlEvent)
			{
				this.UserControlEvent(this, new UserControlEventArgs(this.CommandType, _targetDeviceInfo, false));
			}

			this.ResetMessage();
		}

		private void btnDebug_Click(object sender, RoutedEventArgs e)
		{
			if (null != this.UserControlEvent)
			{
				this.UserControlEvent(this, new UserControlEventArgs(UserControlEventType.DetailDebugMessage, _targetDeviceInfo, true));
			}

			this.ResetMessage();
		}

		#endregion UI Event Callbacks
	}
}