﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.NetworkInformation;
using System.Net.Sockets;
using System.Text;
using System.Threading.Tasks;

namespace AirFusion.Common.Network
{
	/// <summary>
	/// Represents the network information.
	/// </summary>
	public class NetworkInfo
	{
		#region Public Properties

		/// <summary>
		/// Gets the network interface.
		/// </summary>
		public NetworkInterface Interface { get; internal set; }

		/// <summary>
		/// Gets the IPV4 address of the network interface.
		/// </summary>
		public IPAddress Address
		{
			get
			{
				if (this.Interface != null && this._address == null)
				{
					this._address = this.Interface.GetIPProperties().UnicastAddresses.Where(info => info.Address.AddressFamily == AddressFamily.InterNetwork).Select(info => info.Address).FirstOrDefault();
				}
				return this._address;
			}
		}

		/// <summary>
		/// Gets the subnet mask of the IPV4 address.
		/// </summary>
		public IPAddress SubnetMask
		{
			get
			{
				if (this.Interface != null && this._subnetMask == null)
				{
					this._subnetMask = this.Interface.GetIPProperties().UnicastAddresses.Where(info => info.Address.AddressFamily == AddressFamily.InterNetwork).Select(info => info.IPv4Mask).FirstOrDefault();
				}
				return this._subnetMask;
			}
		}

		/// <summary>
		/// Indicates if the network interface is wireless.
		/// </summary>
		public bool IsWireless
		{
			get
			{
				return this.Interface != null ? this.Interface.NetworkInterfaceType == NetworkInterfaceType.Wireless80211 : false;
			}
		}

		#endregion Public Properties

		#region Private/Internal Stuff

		private IPAddress _address = null;
		private IPAddress _subnetMask = null;

		internal NetworkInfo(NetworkInterface networkInterface)
		{
			this.Interface = networkInterface;
		}

		#endregion Private/Internal Stuff
	}
}