﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.NetworkInformation;
using System.Net.Sockets;
using System.Text;
using System.Threading.Tasks;

namespace AirFusion.Common.Network
{
	/// <summary>
	/// Represents the wireless network information.
	/// </summary>
	public class WirelessNetworkInfo : NetworkInfo
	{
		#region Public Properties

		/// <summary>
		/// Indicates whether the access point is hidden.
		/// </summary>
		public bool IsHidden { get; set; }

		/// <summary>
		/// Gets the SSID of the connected wireless AP.
		/// </summary>
		public string Ssid { get; internal set; }

		/// <summary>
		/// The BSSID of the connected wireless AP.
		/// </summary>
		public byte[] Bssid { get; internal set; }

		/// <summary>
		/// Gets the BSSID string of the connected wireless AP.
		/// </summary>
		public string BssidString
		{
			get
			{
				if (this._bssidString == null)
				{
					if (this.Bssid == null)
					{
						return null;
					}

					this._bssidString = string.Empty;
					for (int i = 0; i < this.Bssid.Length; ++i)
					{
						this._bssidString += this.Bssid[i].ToString("X2");
						if (i < this.Bssid.Length - 1)
						{
							this._bssidString += ":";
						}
					}
				}

				return this._bssidString;
			}
		}

		/// <summary>
		/// The encryption of the connected wireless AP.
		/// </summary>
		public WirelessEncryptionProtocolType Encryption { get; internal set; }

		/// <summary>
		/// Gets the password of the connected wireless AP.
		/// </summary>
		public string Password { get; internal set; }

		#endregion Public Properties

		#region Private/Internal Stuff

		private string _bssidString = null;

		/// <summary>
		/// Initializes a new instance of the <see cref="WirelessNetworkInfo"/> class.
		/// </summary>
		/// <param name="networkInterface">The network interface.</param>
		internal WirelessNetworkInfo(NetworkInterface networkInterface)
			: base(networkInterface)
		{
			this.IsHidden = false;
			this.Ssid = null;
			this.Bssid = null;
			this.Encryption = WirelessEncryptionProtocolType.None;
			this.Password = null;
		}

		#endregion Private/Internal Stuff
	}
}