﻿using System;

namespace AirFusion.Common.Wmi.Cim
{
	/// <summary>
	/// The CIM_ManagedSystemElement class is the base class for the system element hierarchy. Any distinguishable system component is a candidate for inclusion in this class.
	/// Examples include software components, such as files; devices, such as disk drives and controllers; and physical components, such as chips and cards.
	/// </summary>
	[ParameterValue("CIM_ManagedSystemElement")]
	public class WmiManagedSystemElement
	{
		/// <summary>
		/// Gets the short textual description of the object.
		/// </summary>
		[ParameterValue("Caption")]
		public string Caption { get; internal set; }

		/// <summary>
		/// Gets the textual description of the object.
		/// </summary>
		[ParameterValue("Description")]
		public string Description { get; internal set; }

		/// <summary>
		/// Gets the date and time the object was installed. This property does not need a value to indicate that the object is installed.
		/// </summary>
		[ParameterValue("InstallDate"), TypeConverterTypeAttribute(typeof(Converters.WmiDateTimeConverter))]
		public DateTime? InstallDate { get; internal set; }

		/// <summary>
		/// Gets the label by which the object is known. When subclassed, this property can be overridden to be a key property.
		/// </summary>
		[ParameterValue("Name")]
		public string Name { get; internal set; }

		/// <summary>
		/// Gets the string that indicates the current status of the object. Operational and non-operational status can be defined.
		/// Operational status can include "OK", "Degraded", and "Pred Fail".
		/// "Pred Fail" indicates that an element is functioning properly, but is predicting a failure (for example, a SMART-enabled hard disk drive).
		/// Non-operational status can include "Error", "Starting", "Stopping", and "Service".
		/// "Service" can apply during disk mirror-resilvering, reloading a user permissions list, or other administrative work. Not all such work is online,
		/// but the managed element is neither "OK" nor in one of the other states.
		/// </summary>
		[ParameterValue("Status")]
		public string Status { get; internal set; }

		/// <summary>
		/// Initializes the object.
		/// </summary>
		public WmiManagedSystemElement()
		{
			this.Caption = null;
			this.Description = null;
			this.InstallDate = null;
			this.Name = null;
			this.Status = null;
		}
	}
}