﻿using System;

namespace AirFusion.Common.Wmi.Win32
{
	/// <summary>
	/// The Win32_ComputerSystem WMI class represents a computer system running Windows.
	/// </summary>
	[ParameterValue("Win32_ComputerSystem")]
	public class WmiComputerSystem : Cim.WmiUnitaryComputerSystem
	{
		/// <summary>
		/// Gets the system hardware security settings for administrator password status.
		/// </summary>
		[ParameterValue("AdminPasswordStatus")]
		public UInt16? AdminPasswordStatus { get; internal set; }

		/// <summary>
		/// Gets a value that indicates if the system manages the page file.
		/// </summary>
		/// <remarks>
		/// Windows Server 2003: This property is not available.
		/// </remarks>
		[ParameterValue("AutomaticManagedPagefile")]
		public bool? AutomaticManagedPagefile { get; internal set; }

		/// <summary>
		/// Gets a value that indicates if the automatic reset boot option is enabled.
		/// </summary>
		[ParameterValue("AutomaticResetBootOption")]
		public bool? AutomaticResetBootOption { get; internal set; }

		/// <summary>
		/// Gets a value that indicates if the automatic reset is enabled.
		/// </summary>
		[ParameterValue("AutomaticResetCapability")]
		public bool? AutomaticResetCapability { get; internal set; }

		/// <summary>
		/// Gets the boot option limit.
		/// </summary>
		[ParameterValue("BootOptionOnLimit")]
		public UInt16? BootOptionOnLimit { get; internal set; }

		/// <summary>
		/// Gets the type of reboot action after the time on the watchdog timer is elapsed.
		/// </summary>
		[ParameterValue("BootOptionOnWatchDog")]
		public UInt16? BootOptionOnWatchDog { get; internal set; }

		/// <summary>
		/// Gets a value that indicates if a boot ROM is supported.
		/// </summary>
		[ParameterValue("BootROMSupported")]
		public bool? BootROMSupported { get; internal set; }

		/// <summary>
		/// Gets the bootup state.
		/// </summary>
		/// <example>
		/// Normal boot
		/// </example>
		/// <example>
		/// Fail-safe boot
		/// </example>
		[ParameterValue("BootupState")]
		public string BootupState { get; internal set; }

		/// <summary>
		/// Gets the boot up state of the chassis.
		/// </summary>
		[ParameterValue("ChassisBootupState")]
		public UInt16? ChassisBootupState { get; internal set; }

		/// <summary>
		/// Gets the amount of time the unitary computer system is offset from Coordinated Universal Time (UTC).
		/// </summary>
		[ParameterValue("CurrentTimeZone")]
		public Int16? CurrentTimeZone { get; internal set; }

		/// <summary>
		/// Gets a value that indicates if the daylight savings mode is ON.
		/// </summary>
		[ParameterValue("DaylightInEffect")]
		public bool? DaylightInEffect { get; internal set; }

		/// <summary>
		/// Gets the name of local computer according to the domain name server (DNS).
		/// </summary>
		[ParameterValue("DNSHostName")]
		public string DNSHostName { get; internal set; }

		/// <summary>
		/// Gets the name of the domain to which a computer belongs.
		/// </summary>
		/// <remarks>If the computer is not part of a domain, then the name of the workgroup is returned.</remarks>
		[ParameterValue("Domain")]
		public string Domain { get; internal set; }

		/// <summary>
		/// Gets the role of a computer in an assigned domain workgroup. A domain workgroup is a collection of computers on the same network. For example, a DomainRole property may show that a computer is a member workstation.
		/// </summary>
		[ParameterValue("DomainRole")]
		public UInt16? DomainRole { get; internal set; }

		/// <summary>
		/// Gets the daylight savings time (DST) on a computer.
		/// A value of True indicates that the system time changes to an hour ahead or behind when DST starts or ends.
		/// A value of False indicates that the system time does not change to an hour ahead or behind when DST starts or ends.
		/// A value of NULL indicates that the DST status is unknown on a system.
		/// </summary>
		[ParameterValue("EnableDaylightSavingsTime")]
		public bool? EnableDaylightSavingsTime { get; internal set; }

		/// <summary>
		/// Gets the hardware security settings for the reset button on a computer.
		/// </summary>
		[ParameterValue("FrontPanelResetStatus")]
		public UInt16? FrontPanelResetStatus { get; internal set; }

		/// <summary>
		/// Gets a value that indicates if a hypervisor is present.
		/// </summary>
		[ParameterValue("HypervisorPresent")]
		public bool? HypervisorPresent { get; internal set; }

		/// <summary>
		/// Gets a value that indicates if an infrared (IR) port exists on a computer system.
		/// </summary>
		[ParameterValue("InfraredSupported")]
		public bool? InfraredSupported { get; internal set; }

		/// <summary>
		/// Gets the system hardware security settings for Keyboard Password Status.
		/// </summary>
		[ParameterValue("KeyboardPasswordStatus")]
		public UInt16? KeyboardPasswordStatus { get; internal set; }

		/// <summary>
		/// Gets the name of a computer manufacturer.
		/// </summary>
		[ParameterValue("Manufacturer")]
		public string Manufacturer { get; internal set; }

		/// <summary>
		/// Gets the product name that a manufacturer gives to a computer. This property must have a value.
		/// </summary>
		[ParameterValue("Model")]
		public string Model { get; internal set; }

		/// <summary>
		/// Gets a value that indicates if the network Server Mode is enabled.
		/// </summary>
		[ParameterValue("NetworkServerModeEnabled")]
		public bool? NetworkServerModeEnabled { get; internal set; }

		/// <summary>
		/// Gets the number of logical processors available on the computer.
		/// You can use NumberOfLogicalProcessors and NumberOfProcessors to determine if the computer is hyperthreading.
		/// </summary>
		/// <remarks>
		/// Windows Server 2003: This property is not available.
		/// </remarks>
		[ParameterValue("NumberOfLogicalProcessors")]
		public UInt32? NumberOfLogicalProcessors { get; internal set; }

		/// <summary>
		/// Gets the number of physical processors currently available on a system. This is the number of enabled processors for a system, which does not include the disabled processors.
		/// If a computer system has two physical processors each containing two logical processors, then the value of NumberOfProcessors is 2 and NumberOfLogicalProcessors is 4.
		/// The processors may be multicore or they may be hyperthreading processors.
		/// </summary>
		/// <remarks>
		/// Windows Server 2003: Because the NumberOfLogicalProcessors property is not available, NumberOfProcessors indicates the number of logical processors available in the system.
		/// In the case of a computer system that has two physical processors each containing two logical processors, the value of NumberOfProcessors is 4.
		/// </remarks>
		[ParameterValue("NumberOfProcessors")]
		public UInt32? NumberOfProcessors { get; internal set; }

		/// <summary>
		/// Gets the list of data for a bitmap that the original equipment manufacturer (OEM) creates.
		/// </summary>
		[ParameterValue("OEMLogoBitmap")]
		public byte[] OEMLogoBitmap { get; internal set; }

		/// <summary>
		/// Gets the list of free-form strings that an OEM defines.
		/// For example, an OEM defines the part numbers for system reference documents, manufacturer contact information, and so on.
		/// </summary>
		[ParameterValue("OEMStringArray")]
		public string[] OEMStringArray { get; internal set; }

		/// <summary>
		/// Gets a value that indicates if the computer is part of a domain. If the value is NULL, the computer is not in a domain or the status is unknown.
		/// If you remove the computer from a domain, the value becomes false.
		/// </summary>
		[ParameterValue("PartOfDomain")]
		public bool? PartOfDomain { get; internal set; }

		/// <summary>
		/// Gets the time delay before a reboot is initiated—in milliseconds. It is used after a system power cycle, local or remote system reset, and automatic system reset.
		/// A value of –1 (minus one) indicates that the pause value is unknown.
		/// </summary>
		[ParameterValue("PauseAfterReset")]
		public Int64? PauseAfterReset { get; internal set; }

		/// <summary>
		/// Gets the type of the computer in use, such as laptop, desktop, or Tablet.
		/// </summary>
		/// <remarks>
		/// Windows Server 2003: This property is not available.
		/// </remarks>
		[ParameterValue("PCSystemType")]
		public UInt16? PCSystemType { get; internal set; }

		/// <summary>
		/// Gets the system hardware security settings for Power-On Password Status.
		/// </summary>
		[ParameterValue("PowerOnPasswordStatus")]
		public UInt16? PowerOnPasswordStatus { get; internal set; }

		/// <summary>
		/// Gets the state of the power supply or supplies when last booted. The following list identifies the values for this property.
		/// </summary>
		[ParameterValue("PowerSupplyState")]
		public UInt16? PowerSupplyState { get; internal set; }

		/// <summary>
		/// Gets the number of automatic resets since the last reset. A value of –1 (minus one) indicates that the count is unknown.
		/// </summary>
		[ParameterValue("ResetCount")]
		public Int16? ResetCount { get; internal set; }

		/// <summary>
		/// Gets the number of consecutive times a system reset is attempted. A value of –1 (minus one) indicates that the limit is unknown.
		/// </summary>
		[ParameterValue("ResetLimit")]
		public Int16? ResetLimit { get; internal set; }

		/// <summary>
		/// Gets the list of the support contact information for the Windows operating system.
		/// </summary>
		[ParameterValue("SupportContactDescription")]
		public string SupportContactDescription { get; internal set; }

		/// <summary>
		/// SystemStartupDelay is no longer available for use because Boot.ini is not used to configure system startup.
		/// Instead, use the BCD classes supplied by the Boot Configuration Data (BCD) WMI provider or the Bcdedit command.
		/// </summary>
		/// <remarks>
		/// Windows Server 2003: Gets the time to delay before starting the Windows operating system—in seconds.
		/// You can use SystemStartupDelay with SystemStartupSetting and SystemStartupOptions to modify Boot.ini system startup settings.
		/// </remarks>
		[ParameterValue("SystemStartupDelay")]
		public UInt16? SystemStartupDelay { get; internal set; }

		/// <summary>
		/// SystemStartupOptions is no longer available for use because Boot.ini is not used to configure system startup.
		/// Instead, use the BCD classes supplied by the Boot Configuration Data (BCD) WMI provider or the Bcdedit command.
		/// </summary>
		/// <remarks>
		/// Windows Server 2003: Gets the list of the options for starting up the computer system running Windows.
		/// This property must have a value. You can use SystemStartupOptions with SystemStartupSetting and SystemStartupDelay to modify Boot.ini system startup settings.
		/// </remarks>
		[ParameterValue("SystemStartupOptions")]
		public string SystemStartupOptions { get; internal set; }

		/// <summary>
		/// SystemStartupSetting is no longer available for use because Boot.ini is not used to configure system startup.
		/// Instead, use the BCD classes supplied by the Boot Configuration Data (BCD) WMI provider or the Bcdedit command.
		/// </summary>
		/// <remarks>
		/// Windows Server 2003: Gets the index of the default startup profile.
		/// This value is calculated so that it usually returns 0 (zero), because at write-time the profile string is physically moved to the top of the list,
		/// which is how Windows determines the default value.
		/// You can use SystemStartupSetting with SystemStartupDelay and SystemStartupOptions to modify Boot.ini system startup settings.
		/// </remarks>
		[ParameterValue("SystemStartupSetting")]
		public byte? SystemStartupSetting { get; internal set; }

		/// <summary>
		/// Gets the system running on the Windows-based computer. This property must have a value.
		/// </summary>
		[ParameterValue("SystemType")]
		public string SystemType { get; internal set; }

		/// <summary>
		/// Gets the thermal state of the system when last booted.
		/// </summary>
		[ParameterValue("ThermalState")]
		public UInt16? ThermalState { get; internal set; }

		/// <summary>
		/// Gets the total size of physical memory. Be aware that, under some circumstances, this property may not return an accurate value for the physical memory.
		/// For example, it is not accurate if the BIOS is using some of the physical memory. For an accurate value, use the Capacity property in Win32_PhysicalMemory instead.
		/// </summary>
		[ParameterValue("TotalPhysicalMemory")]
		public UInt64? TotalPhysicalMemory { get; internal set; }

		/// <summary>
		/// Gets the name of a user that is logged on currently. This property must have a value.
		/// In a terminal services session, UserName returns the name of the user that is logged on to the console—not the user logged on during the terminal service session.
		/// </summary>
		[ParameterValue("UserName")]
		public string UserName { get; internal set; }

		/// <summary>
		/// Gets the event that causes the system to power up.
		/// </summary>
		[ParameterValue("WakeUpType")]
		public UInt16? WakeUpType { get; internal set; }

		/// <summary>
		/// Gets the name of the workgroup for this computer. If the value of the PartOfDomain property is False, then the name of the workgroup is returned.
		/// </summary>
		[ParameterValue("Workgroup")]
		public string Workgroup { get; internal set; }

		/// <summary>
		/// Initializes the object.
		/// </summary>
		public WmiComputerSystem()
		{
			this.AdminPasswordStatus = null;
			this.AutomaticManagedPagefile = null;
			this.AutomaticResetBootOption = null;
			this.AutomaticResetCapability = null;
			this.BootOptionOnLimit = null;
			this.BootOptionOnWatchDog = null;
			this.BootROMSupported = null;
			this.BootupState = null;
			this.ChassisBootupState = null;
			this.CurrentTimeZone = null;
			this.DaylightInEffect = null;
			this.Description = null;
			this.DNSHostName = null;
			this.Domain = null;
			this.DomainRole = null;
			this.EnableDaylightSavingsTime = null;
			this.FrontPanelResetStatus = null;
			this.InfraredSupported = null;
			this.KeyboardPasswordStatus = null;
			this.Manufacturer = null;
			this.Model = null;
			this.NetworkServerModeEnabled = null;
			this.NumberOfLogicalProcessors = null;
			this.NumberOfProcessors = null;
			this.OEMLogoBitmap = null;
			this.OEMStringArray = null;
			this.PartOfDomain = null;
			this.PauseAfterReset = null;
			this.PCSystemType = null;
			this.PowerOnPasswordStatus = null;
			this.PowerSupplyState = null;
			this.ResetCount = null;
			this.ResetLimit = null;
			this.SupportContactDescription = null;
			this.SystemStartupDelay = null;
			this.SystemStartupOptions = null;
			this.SystemStartupSetting = null;
			this.SystemType = null;
			this.ThermalState = null;
			this.TotalPhysicalMemory = null;
			this.UserName = null;
			this.WakeUpType = null;
			this.Workgroup = null;
		}
	}
}