﻿using Newtonsoft.Json;
using System;

namespace AirFusion.Common.JsonConverters
{
	internal class ByteArrayConverter : JsonConverter
	{
		public override bool CanConvert(Type objectType)
		{
			return typeof(byte[]) == objectType;
		}

		public override object ReadJson(JsonReader reader, Type objectType, object existingValue, JsonSerializer serializer)
		{
			if (CanConvert(objectType) == false)
			{
				throw new ArgumentException("Not supported object type: " + objectType.GetFullName());
			}

			string stringValue = reader.Value != null ? reader.Value.ToString() : null;

			switch (reader.TokenType)
			{
				case JsonToken.Null:
					return null;

				case JsonToken.String:
					if (!string.IsNullOrEmpty(stringValue))
					{
						return stringValue.FromHexString<byte[]>();
					}
					break;

				default:
					throw new ArgumentException("Invalid token type.");
			}

			return null;
		}

		public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer)
		{
			if (value is byte[])
			{
				writer.WriteValue(((byte[])value).ToHexString(""));
			}
			else
			{
				writer.WriteNull();
			}
		}
	}
}
