﻿using AirFusion.Common;
using System.Net.Sockets;
using System.Text;

namespace AirFusion.Manager.DeviceManager
{
	public partial class DnsSdHandler
	{
		/// <summary>
		/// Represents the data of a address querier.
		/// </summary>
		public class AddrQuerierData : ProcessorData
		{
			/// <summary>
			/// Gets the hostname.
			/// </summary>
			public string Hostname { get; internal set; }

			/// <summary>
			/// Gets the index of the interface.
			/// </summary>
			public uint InterfaceIndex { get; internal set; }

			/// <summary>
			/// Gets the address family of the querier.
			/// </summary>
			public AddressFamily AddressFamily { get; internal set; }

			/// <summary>
			/// Gets the query method.
			/// </summary>
			public AddrQueryMethod QueryMethod { get; internal set; }

			/// <summary>
			/// Gets the callback function.
			/// </summary>
			internal AddressQuerierCallback Callback { get; set; }

			/// <summary>
			/// Initializes a new instance of the <see cref="AddrQuerierData"/> class.
			/// </summary>
			internal AddrQuerierData()
			{
				this.Hostname = null;
				this.InterfaceIndex = 0;
				this.AddressFamily =  AddressFamily.Unknown;
				this.QueryMethod = AddrQueryMethod.Default;
				this.Callback = null;
			}

			/// <summary>
			/// Initializes a new instance of the <see cref="AddrQuerierData"/> class.
			/// </summary>
			/// <param name="hostname">The hostname.</param>
			/// <param name="interfaceIndex">The index of the interface.</param>
			/// <param name="addressFamily">The address family.</param>
			/// <param name="queryMethod">The query method.</param>
			/// <param name="callback">The callback.</param>
			internal AddrQuerierData(string hostname, uint interfaceIndex, AddressFamily addressFamily, AddrQueryMethod queryMethod, AddressQuerierCallback callback)
			{
				this.Hostname = hostname;
				this.InterfaceIndex = interfaceIndex;
				this.AddressFamily = addressFamily;
				this.QueryMethod = queryMethod;
				this.Callback = callback;
			}

			/// <summary>
			/// Returns a <see cref="System.String" /> that represents this instance.
			/// </summary>
			/// <returns>
			/// A <see cref="System.String" /> that represents this instance.
			/// </returns>
			public override string ToString()
			{
				StringBuilder builder = new StringBuilder();
				builder.AppendLine(this.GetType().GetFullName() + " (Base):\r\n" + base.ToString());
				builder.AppendLine(this.GetType().GetFullName() + ".Hostname: " + this.Hostname);
				builder.AppendLine(this.GetType().GetFullName() + ".InterfaceIndex: " + this.InterfaceIndex);
				builder.AppendLine(this.GetType().GetFullName() + ".AddressFamily: " + this.AddressFamily);
				builder.AppendLine(this.GetType().GetFullName() + ".QueryMethod: " + this.QueryMethod);

				return builder.ToString().TrimEnd();
			}
		}
	}
}
