﻿using AirFusion.Common;
using System.Text;

namespace AirFusion.Manager.DeviceManager
{
	public partial class DnsSdHandler
	{
		/// <summary>
		/// Represents the data of a browser.
		/// </summary>
		public class BrowserData : ProcessorData
		{
			/// <summary>
			/// Gets the index of the interface.
			/// </summary>
			public uint InterfaceIndex { get; internal set; }

			/// <summary>
			/// Gets the callback function.
			/// </summary>
			internal BrowserCallback Callback { get; set; }

			/// <summary>
			/// Initializes a new instance of the <see cref="BrowserData"/> class.
			/// </summary>
			internal BrowserData()
			{
				this.InterfaceIndex = 0;
				this.Callback = null;
			}

			/// <summary>
			/// Initializes a new instance of the <see cref="BrowserData"/> class.
			/// </summary>
			/// <param name="interfaceIndex">The index of the interface.</param>
			/// <param name="callback">The callback.</param>
			internal BrowserData(uint interfaceIndex, BrowserCallback callback)
			{
				this.InterfaceIndex = interfaceIndex;
				this.Callback = callback;
			}

			/// <summary>
			/// Returns a <see cref="System.String" /> that represents this instance.
			/// </summary>
			/// <returns>
			/// A <see cref="System.String" /> that represents this instance.
			/// </returns>
			public override string ToString()
			{
				StringBuilder builder = new StringBuilder();
				builder.AppendLine(this.GetType().GetFullName() + " (Base):\r\n" + base.ToString());
				builder.AppendLine(this.GetType().GetFullName() + ".InterfaceIndex: " + this.InterfaceIndex);

				return builder.ToString().TrimEnd();
			}
		}
	}
}
