﻿using AirFusion.Common;
using System;
using System.Text;

namespace AirFusion.Manager.DeviceManager
{
	public partial class DnsSdHandler
	{
		/// <summary>
		/// Represents the base class of callback arguments.
		/// </summary>
		public abstract class BaseCallbackArgs
		{
			/// <summary>
			/// Gets the bundle <see cref="DnsSdHandler"/> instance.
			/// </summary>
			public DnsSdHandler Handler { get; internal set; }

			/// <summary>
			/// Gets the processor ID.
			/// </summary>
			public uint ProcessorID { get; internal set; }

			/// <summary>
			/// Gets the exception if any.
			/// </summary>
			public Exception Exception { get; internal set; }

			/// <summary>
			/// Gets a value indicating whether this operation is completed.
			/// </summary>
			public bool IsCompleted { get; internal set; }

			/// <summary>
			/// Gets a value indicating whether this operation is faulted.
			/// </summary>
			public bool IsFaulted
			{
				get
				{
					return this.Exception != null;
				}
			}

			/// <summary>
			/// Initializes a new instance of the <see cref="BaseCallbackArgs"/> class.
			/// </summary>
			internal BaseCallbackArgs()
			{
				this.Handler = null;
				this.ProcessorID = 0;
				this.Exception = null;
				this.IsCompleted = false;
			}

			/// <summary>
			/// Returns a <see cref="System.String" /> that represents this instance.
			/// </summary>
			/// <returns>
			/// A <see cref="System.String" /> that represents this instance.
			/// </returns>
			public override string ToString()
			{
				StringBuilder builder = new StringBuilder();
				builder.AppendLine(this.GetType().GetFullName() + ".ProcessorID: " + this.ProcessorID);
				builder.AppendLine(this.GetType().GetFullName() + ".Exception: " + (this.Exception != null ? "\r\n" + CommonUtils.GetExceptionMessages(this.Exception) : "N/A"));
				builder.AppendLine(this.GetType().GetFullName() + ".IsCompleted: " + this.IsCompleted);

				return builder.ToString().TrimEnd();
			}
		}
	}
}
