﻿using AirFusion.Common;
using System;
using System.Text;

namespace AirFusion.Manager.DeviceManager
{
	/// <summary>
	/// Represents the event arguments of discovery event.
	/// </summary>
	public sealed class DiscoveryEventArgs : EventArgs
	{
		#region Enumerations

		/// <summary>
		/// Specifies the type of discovery event.
		/// </summary>
		public enum DiscoveryEventType
		{
			/// <summary>
			/// Indicates the device is found or updated.
			/// </summary>
			DeviceFoundOrUpdate,

			/// <summary>
			/// Indicates the device is lost.
			/// </summary>
			DeviceLost,

			/// <summary>
			/// Indicates an unknown event.
			/// </summary>
			Unknown = -1
		}

		#endregion Enumerations

		#region Properties

		/// <summary>
		/// Gets the device.
		/// </summary>
		public AFUserDevice Device { get; private set; }

		/// <summary>
		/// Gets the type of the event.
		/// </summary>
		public DiscoveryEventType EventType { get; private set; }

		#endregion Properties

		#region Methods

		/// <summary>
		/// Initializes a new instance of the <see cref="DiscoveryEventArgs"/> class.
		/// </summary>
		internal DiscoveryEventArgs()
		{
			this.Device = null;
			this.EventType = DiscoveryEventType.Unknown;
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="DiscoveryEventArgs" /> class.
		/// </summary>
		/// <param name="device">The device.</param>
		/// <param name="eventType">The type of the event.</param>
		internal DiscoveryEventArgs(AFUserDevice device, DiscoveryEventType eventType)
		{
			this.Device = device;
			this.EventType = eventType;
		}

		/// <summary>
		/// Returns a <see cref="System.String" /> that represents this instance.
		/// </summary>
		/// <returns>
		/// A <see cref="System.String" /> that represents this instance.
		/// </returns>
		public override string ToString()
		{
			StringBuilder builder = new StringBuilder();
			builder.AppendLine(this.GetType().GetFullName() + ".Device" + (this.Device != null ? ".InstanceID: " + this.Device.InstanceID.GetStringFormat() : ": N/A"));
			builder.AppendLine(this.GetType().GetFullName() + ".EventType: " + this.EventType);

			return builder.ToString().TrimEnd();
		}

		#endregion Methods
	}
}