﻿using AirFusion.Common;
using System;
using System.Text;

namespace AirFusion.Manager.DeviceManager
{
	/// <summary>
	/// Represents the event arguments of error event.
	/// </summary>
	public sealed class ErrorEventArgs : EventArgs
	{
		#region Properties

		/// <summary>
		/// Gets the error code.
		/// </summary>
		public ErrorType ErrorCode { get; private set; }

		/// <summary>
		/// Gets the exception if any.
		/// </summary>
		public Exception Exception { get; private set; }

		/// <summary>
		/// Gets the device of the error event if any.
		/// </summary>
		public AFUserDevice Device { get; private set; }

		#endregion Properties

		#region Methods

		/// <summary>
		/// Initializes a new instance of the <see cref="ErrorEventArgs"/> class.
		/// </summary>
		internal ErrorEventArgs()
		{
			this.ErrorCode = ErrorType.UnknownError;
			this.Exception = null;
			this.Device = null;
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="ErrorEventArgs" /> class.
		/// </summary>
		/// <param name="errorCode">The error code.</param>
		/// <param name="exception">The exception if any.</param>
		/// <param name="device">The device of the error event if any.</param>
		internal ErrorEventArgs(ErrorType errorCode, Exception exception = null, AFUserDevice device = null)
		{
			this.ErrorCode = errorCode;
			this.Exception = exception;
			this.Device = device;
		}

		/// <summary>
		/// Returns a <see cref="System.String" /> that represents this instance.
		/// </summary>
		/// <returns>
		/// A <see cref="System.String" /> that represents this instance.
		/// </returns>
		public override string ToString()
		{
			StringBuilder builder = new StringBuilder();
			builder.AppendLine(this.GetType().GetFullName() + ".ErrorCode: " + this.ErrorCode);
			builder.AppendLine(this.GetType().GetFullName() + ".Exception: " + (this.Exception != null ? "\r\n" + CommonUtils.GetExceptionMessages(this.Exception) : "N/A"));
			builder.AppendLine(this.GetType().GetFullName() + ".Device" + (this.Device != null ? ".InstanceID: " + this.Device.InstanceID.GetStringFormat() : ": N/A"));

			return builder.ToString().TrimEnd();
		}

		#endregion Methods
	}
}