﻿using System.Net.Http;

namespace AirFusion.WebService
{
	/// <summary>
	/// Represents a web service request filter.
	/// </summary>
	public interface IWebServiceRequestFilter
	{
		/// <summary>
		/// Filters a request.
		/// </summary>
		/// <param name="requestMessage">The request message, any modification could be made to this instance.</param>
		/// <param name="responseMessage">The response message, if any.</param>
		/// <returns>True if to continue next filters.</returns>
		/// <remarks>
		/// - The exception thrown by this method should be considered as an invalid request and propagate to the origin if it's requested by local.
		/// - The exception thrown by this method should be considered as an invalid request and response to the remote if it's requested by remote.
		/// - If the output 'responseMessage' is not null, the request will be responsed with the specified 'responseMessage'.
		/// - The 'responseMessage' should be always null if it's requested by local.
		/// </remarks>
		bool Filter(HttpRequestMessage requestMessage, out HttpResponseMessage responseMessage);
	}
}
