﻿using AirFusion.Common;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading.Tasks;

namespace AirFusion.WebService
{
	public sealed partial class WebServicesManager
	{
		/// <summary>
		/// Represents the raw request.
		/// </summary>
		public sealed class HttpRemoteRequest
		{
			/// <summary>
			/// Gets the ip address of client.
			/// </summary>
			public IPAddress ClientIP { get; internal set; }

			/// <summary>
			/// Gets the port of client.
			/// </summary>
			public ushort Port { get; internal set; }

			/// <summary>
			/// Gets the request message.
			/// </summary>
			public HttpRemoteRequestMessage RequestMessage { get; internal set; }

			/// <summary>
			/// Initializes a new instance of the <see cref="HttpRemoteRequest"/> class.
			/// </summary>
			internal HttpRemoteRequest()
			{
				this.ClientIP = null;
				this.Port = ushort.MinValue;
				this.RequestMessage = null;
			}

			/// <summary>
			/// Returns a <see cref="System.String" /> that represents this instance.
			/// </summary>
			/// <returns>
			/// A <see cref="System.String" /> that represents this instance.
			/// </returns>
			public override string ToString()
			{
				var builder = new StringBuilder();
				builder.AppendLine(this.GetType().GetFullName() + ".ClientIP: " + (this.ClientIP != null ? this.ClientIP.ToString() : "N/A"));
				builder.AppendLine(this.GetType().GetFullName() + ".Port: " + this.Port);
				builder.AppendLine(this.GetType().GetFullName() + ".RequestMessage: " + (this.RequestMessage != null ? "\r\n" + this.RequestMessage.ToString() : "N/A"));

				return builder.ToString().TrimEnd();
			}
		}
	}
}
