﻿using AirFusion.Common.Network;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ListWirelessAdapters
{
	internal class Program
	{
		private static void Main(string[] args)
		{
			WlanClient client = new WlanClient();
			var interfaces = client.Interfaces;
			if (interfaces != null && interfaces.Length > 0)
			{
				foreach (var wi in interfaces)
				{
					Console.WriteLine("              GUID : " + wi.InterfaceGuid.ToString());
					Console.WriteLine("              Name : " + wi.InterfaceName);
					Console.WriteLine("       Description : " + wi.InterfaceDescription);
					Console.WriteLine("         Auto Conf : " + wi.Autoconf);
					Console.WriteLine("             State : " + wi.InterfaceState);
					try
					{
						Console.WriteLine("              RSSI : " + wi.RSSI);
					}
					catch { }
					try
					{
						Console.WriteLine("          BSS Type : " + wi.BssType);
					}
					catch { }
					try
					{
						Console.WriteLine("           Channel : " + wi.Channel);
					}
					catch { }
					try
					{
						Console.WriteLine("    Operation Mode : " + wi.CurrentOperationMode);
					}
					catch { }
					Console.WriteLine();
					Console.WriteLine("Current Connection :");
					try
					{
						Console.WriteLine("                 State : " + wi.CurrentConnection.isState);
						Console.WriteLine("          Profile Name : " + wi.CurrentConnection.profileName);
						Console.WriteLine("       Connection Mode : " + wi.CurrentConnection.wlanConnectionMode);
						Console.WriteLine("            Asso. SSID : " + GetSsidString(wi.CurrentConnection.wlanAssociationAttributes.dot11Ssid));
						Console.WriteLine("           Asso. BSSID : " + wi.CurrentConnection.wlanAssociationAttributes.Dot11Bssid.ToString());
						Console.WriteLine("        Asso. BSS Type : " + wi.CurrentConnection.wlanAssociationAttributes.dot11BssType);
						Console.WriteLine("        Asso.PHY Index : " + wi.CurrentConnection.wlanAssociationAttributes.dot11PhyIndex);
						Console.WriteLine("        Asso. PHY Type : " + wi.CurrentConnection.wlanAssociationAttributes.dot11PhyType);
						Console.WriteLine("         Asso. RX Rate : " + wi.CurrentConnection.wlanAssociationAttributes.rxRate);
						Console.WriteLine("         Asso. TX Rate : " + wi.CurrentConnection.wlanAssociationAttributes.txRate);
						Console.WriteLine("  Asso. Signal Quality : " + wi.CurrentConnection.wlanAssociationAttributes.wlanSignalQuality);
						Console.WriteLine("        Auth Algorithm : " + wi.CurrentConnection.wlanSecurityAttributes.dot11AuthAlgorithm);
						Console.WriteLine("      Cipher Algorithm : " + wi.CurrentConnection.wlanSecurityAttributes.dot11CipherAlgorithm);
						Console.WriteLine("       Encryption Type : " + GetEncryption(wi.CurrentConnection.wlanSecurityAttributes.dot11AuthAlgorithm, wi.CurrentConnection.wlanSecurityAttributes.dot11CipherAlgorithm));
						Console.WriteLine("         One X Enabled : " + wi.CurrentConnection.wlanSecurityAttributes.oneXEnabled);
						Console.WriteLine("      Security Enabled : " + wi.CurrentConnection.wlanSecurityAttributes.securityEnabled);
						Console.WriteLine();
						Console.WriteLine("       Profile XML :");
						Console.WriteLine(wi.GetProfileXml(wi.CurrentConnection.profileName));
					}
					catch { }
				}
			}
			else
			{
				Console.WriteLine("There's no wireless interface.");
			}

			Console.ReadKey();
		}

		private static string GetSsidString(Wlan.Dot11Ssid ssid)
		{
			if (ssid.SSID == null || ssid.SSIDLength == 0 || ssid.SSID.Length == 0)
			{
				return string.Empty;
			}

			for (int i = 0; i < ssid.SSIDLength; ++i)
			{
				if (ssid.SSID[i] == 0)
				{
					return string.Empty; // invalid byte array
				}
			}

			return Encoding.UTF8.GetString(ssid.SSID, 0, (int)ssid.SSIDLength);
		}

		private static string GetEncryption(Wlan.Dot11AuthAlgorithm auth, Wlan.Dot11CipherAlgorithm cipher)
		{
			switch (auth)
			{
				case Wlan.Dot11AuthAlgorithm.WPA:
				case Wlan.Dot11AuthAlgorithm.WPA_PSK:
				case Wlan.Dot11AuthAlgorithm.WPA_None:
					return "WPA";

				case Wlan.Dot11AuthAlgorithm.RSNA:
				case Wlan.Dot11AuthAlgorithm.RSNA_PSK:
					return "WPA2";

				default:
				case Wlan.Dot11AuthAlgorithm.IEEE80211_Open:
				case Wlan.Dot11AuthAlgorithm.IEEE80211_SharedKey:
					if (cipher != Wlan.Dot11CipherAlgorithm.None)
					{
						return "WEP";
					}
					break;
			}

			return string.Empty;
		}
	}
}