﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace VNC
{
	public partial class VNCClient
	{
		// ---------------------------------------------------------------------------------------------------

		#region Import the Functions of VNCHandler.dll (c++)

		/// <summary>
		/// Define the structure that CreateVNCHandler function will be used
		/// </summary>
		[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
		public struct VNCConnectInfo
		{
			[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 16)]
			public string sIPv4;

			[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 32)]
			public string sPassword;

			public uint uiPortNum;
		}

		#region Functions are used to process VNC handler

		/// <summary>
		/// Create one new VNCHandler
		/// </summary>
		/// <returns>
		/// -1: Failed to create VNCHandler
		/// >0: The index of VNCHandler (unique key, for support multi connection)
		/// </returns>
		[DllImport("InputControlHandler.dll")]
		private static extern int CreateNewVNCHandler();

		/// <summary>
		/// Ask the specified VNC Handle to enter running mode (Make connection with server & monitor the commands that are received from server)
		/// </summary>
		/// <param name="bIndex">The index of VNC Handler</param>
		/// <param name="mConnInfo">Making connection needs information (i.e. IP, Port, and PW)</param>
		/// <returns>0: Success; >0: Failed, Error Code</returns>
		[DllImport("InputControlHandler.dll")]
		private static extern UInt32 RunVNCHandler(byte bIndex, VNCConnectInfo mConnInfo);

		/// <summary>
		/// Initialize the Clipboard function (Set UI handle)
		/// </summary>
		/// <param name="bIndex"></param>
		/// <param name="hWnd"></param>
		/// <returns>True: Successfully; False: Failed</returns>
		[DllImport("InputControlHandler.dll")]
		private static extern bool InitVNCClipboard(byte bIndex, IntPtr hWnd);

		/// <summary>
		/// Removes the window (hWnd) from Clipboard Chain Viewer
		/// </summary>
		/// <param name="bIndex"></param>
		/// <param name="hWnd"></param>
		[DllImport("InputControlHandler.dll")]
		private static extern void RemoveVNCClipboard(IntPtr hWnd);

		/// <summary>
		/// Inform VNC the new screen size in Air Fusion Window
		/// Send the new miracast screen information to VNC Handler (For support coordinate transformation)
		/// </summary>
		/// <param name="bIndex"></param>
		/// <param name="screen_info"></param>
		[DllImport("InputControlHandler.dll")]
		private static extern void SetAFMiracastScreenInfo(byte bIndex, int iWidth, int iHeight, int iDeltaX, int iDeltaY, float fDpiRate);

		/// <summary>
		/// Close and release one VNCHandler (according to the value of VNC Index)
		/// </summary>
		/// <param name="iVncIndex">The index of VNCHandler that get from CreateVNCHandler function</param>
		[DllImport("InputControlHandler.dll")]
		private static extern void CloseVNCHandler(int iVncIndex);

		/// <summary>
		/// Close and release the all VNCHandlers
		/// </summary>
		[DllImport("InputControlHandler.dll")]
		private static extern void ReleaseVNCHandlers();

		/// <summary>
		/// This function is used for handling the windows message of WPF UI
		/// </summary>
		/// <param name="bIndex"></param>
		/// <param name="hwnd"></param>
		/// <param name="msg"></param>
		/// <param name="wParam"></param>
		/// <param name="lParam"></param>
		[DllImport("InputControlHandler.dll")]
		private static extern void VncWinProc(byte bIndex, IntPtr hwnd, int msg, IntPtr wParam, IntPtr lParam);
		
		/// <summary>
		/// Right-clik to paste the clipboard
		/// </summary>
		/// <param name="bIndex"></param>
		[DllImport("InputControlHandler.dll")]
		private static extern void RightClickClipboard(byte bIndex);

		#endregion Functions are used to process VNC handler

		#region Functions are used to send data to phone directly

		/// <summary>
		/// Ask VNC Handler to send the unicode chars to Server
		/// </summary>
		/// <param name="wcBuffer">Unicode string</param>
		/// <param name="iWsLeng">The length of unicode string</param>
		[DllImport("InputControlHandler.dll", CharSet = CharSet.Unicode)]
		private static extern void SendIMEChars(byte bIndex, [MarshalAs(UnmanagedType.LPWStr)]string wsBuffer, int iWsLeng);

		/// <summary>
		/// Send a virtual key code to VNC Server
		/// </summary>
		/// <param name="bIndex"></param>
		/// <param name="uiVirtualKeyCode">Virtual Key Code (Only surpport: VK_BACK = 0x08, VK_RETURN = 0x0D)</param>
		/// <param name="uiKeyStateFlg">KEYEVENTF_KEYDOWN = 0x0000, KEYEVENTF_EXTENDEDKEY = 0x0001, KEYEVENTF_KEYUP = 0x0002,</param>
		[DllImport("InputControlHandler.dll")]
		private static extern void SendVirtualKey(byte bIndex, uint uiVirtualKeyCode, uint uiKeyStateFlg);

		/// <summary>
		/// Sends the touch input event (Down/Up/Move) to VNC Server
		/// </summary>
		/// <param name="bIndex">VNC Client ID</param>
		/// <param name="uiTouchEventType">TouchEventType.Download, TouchEventType.Move, TouchEventType.Up</param>
		/// <param name="point_x">location X coordinate value</param>
		/// <param name="point_y">location Y coordinate value</param>
		[DllImport("InputControlHandler.dll")]
		private static extern void SendTouchControl(byte bIndex, uint uiTouchEventType, Int32 point_x, Int32 point_y);

		#endregion Functions are used to send data to phone directly

		#region Functions are used to handle file transfer

		/// <summary>
		/// Ask VNC Handler to send one local file to Server (ex: Smart Phone)
		/// </summary>
		/// <param name="bIndex">VNC index</param>
		/// <param name="wsClientFilePath">The path of file that will be sent to server</param>
		/// <param name="iFileId"></param>
		[DllImport("InputControlHandler.dll", CharSet = CharSet.Unicode)]
		private static extern void TransferFileToServer(byte bIndex, [MarshalAs(UnmanagedType.LPWStr)]string wsClientFilePath, int iFileId);

		/// <summary>
		/// Specifies the path of VNC download folder
		/// </summary>
		/// <param name="bIndex"></param>
		/// <param name="folder_path"></param>
		[DllImport("InputControlHandler.dll")]
		private static extern void SetDownloadFolderPath(byte bIndex, [MarshalAs(UnmanagedType.LPWStr)] string folder_path);

		#endregion Functions are used to handle file transfer

		#region Functions are used to control VNC debug log

		/// <summary>
		/// Enables the debug log of VNC
		/// </summary>
		/// <param name="logfile_path">Set the path of VNC log file</param>
		/// <param name="iLogLevel">Set the debug level</param>
		[DllImport("InputControlHandler.dll")]
		private static extern void EnableDebugLog([MarshalAs(UnmanagedType.LPStr)] string logfile_path, int iLogLevel);

		/// <summary>
		/// Write message in VNC debug log file
		/// </summary>
		/// <param name="iLogLevel">debugl log level</param>
		/// <param name="sDebugMsg">debug message</param>
		[DllImport("InputControlHandler.dll")]
		private static extern void WriteDebugLog(int iLogLevel, [MarshalAs(UnmanagedType.LPStr)] string sDebugMsg);

		/// <summary>
		/// Disables the debug log of VNC
		/// </summary>
		[DllImport("InputControlHandler.dll")]
		private static extern void CloseDebugLog();

		#endregion Functions are used to control VNC debug log

		#region VNCHandler's Events

		/// <summary>
		/// VNCHandler::ClientConnection returns message (warning or error message)
		/// </summary>
		/// <param name="iVncId"></param>
		/// <param name="iErrorCode"></param>
		[UnmanagedFunctionPointer(CallingConvention.StdCall)]
		private delegate void VNCMessageCallback(int vnc_id, int vnc_msg_type, int vnc_fun_code, int vnc_err_code, [MarshalAs(UnmanagedType.LPStr)] string msg);

		/// <summary>
		/// Set the callback funtion (VNCHandler will use this function to send info to CShap class)
		/// </summary>
		[DllImport("InputControlHandler.dll")]
		private static extern void SetVNCCallbackFun(VNCMessageCallback callbackPointer);

		/// <summary>
		/// VNCHandler::ClientConnection::FileTransfer returns the curren state of uploading one file.
		/// </summary>
		[UnmanagedFunctionPointer(CallingConvention.StdCall)]
		private delegate void FileUploadProCallback(int vnc_id, int file_id, int progress, int state);

		/// <summary>
		/// VNCHandler::ClientConnection::FileTransfer returns the curren state of downloading one file.
		/// </summary>
		[UnmanagedFunctionPointer(CallingConvention.StdCall)]
		private delegate void FileDownloadProCallback(int vnc_id, [MarshalAsAttribute(UnmanagedType.LPWStr)]string file_name, int progress, int state);

		/// <summary>
		/// Set the callback funtion (VNCHandler will use this function to send the state of FileTransfer)
		/// </summary>
		[DllImport("InputControlHandler.dll")]
		private static extern void SetFileTransferCallbackFun(FileUploadProCallback uploadCallback, FileDownloadProCallback downloadCallback);

		#endregion VNCHandler's Events

		#endregion Import the Functions of VNCHandler.dll (c++)

		// Private Members
		private int _vnc_handler_index;

		private bool _bFlag_IsVNCRunning;

		// ---------------------------------------------------------------------------------------------------

		#region Static Members, Events & Functions

		// To store the reference to the callback functions (Let the garbage collector can see these references)
		private static VNCMessageCallback VNCMsgCallback = null;

		private static FileUploadProCallback FTProcCallback = null;
		private static FileDownloadProCallback FTDownloadProcCallback = null;

		public static event VNCMessageEventHandler VNCMessageEvent;

		public static event FileTransferUpdateEventHandler FileTransferUpdateEvent;

		public static void VNCMessage_CallBackFun(int vnc_id, int vnc_msg_type, int vnc_fun_code, int vnc_err_code, string msg)
		{
			VNCMsgItem item = new VNCMsgItem(vnc_id, (VNC_MSG_TYPE)vnc_msg_type, (VNC_FUN_CODE)vnc_fun_code, (VNC_ERR_CODE)vnc_err_code, msg);
			lock (s_oLock_AccessMsgQueue)
			{
				VNCMsgEventQueue.Enqueue(item);
			}
			CreateMonitorMessageThread();
		}

		/// <summary>
		/// This function will be called by VNCHandler (Native Code)
		/// </summary>
		/// <param name="iVncId"></param>
		/// <param name="iFileId"></param>
		/// <param name="iProgress"></param>
		public static void FileUpload_CallBackFun(int iVncId, int iFileId, int iProgress, int state)
		{
			FTUpdateItem tmpFTUpdateRequest = new FTUpdateItem(iVncId, FileTransferType.UploadToServer, iFileId, iProgress, (FileProgressState)state);
			tmpFTUpdateRequest.CurrentCompleteFileCount = s_CurVNCClientsTable[iVncId]._CalcProgressInfo.SendCompleteFilesCount;
			lock (s_oLock_AccessFTPQueue)
			{
				FTProgUpdateEventQueue.Enqueue(tmpFTUpdateRequest);
			}
			CreateMonitorProgressThread();
		}

		public static void FileTransfer_DownloadCallBack(int iVncId, [MarshalAsAttribute(UnmanagedType.LPWStr)]string wsFileName, int iProgress, int state)
		{
			string sDownloadFileName = string.Copy(wsFileName);
			FileTransferType ftType = FileTransferType.DownloadFromServer;
			if (iProgress >= 100)
				ftType = FileTransferType.DownloadFileFinish;

			FTUpdateItem tmpFTUpdateRequest = new FTUpdateItem(iVncId, ftType, sDownloadFileName, iProgress, (FileProgressState)state);
			tmpFTUpdateRequest.CurrentCompleteFileCount = 0;
			lock (s_oLock_AccessFTPQueue)
			{
				FTProgUpdateEventQueue.Enqueue(tmpFTUpdateRequest);
			}
			CreateMonitorProgressThread();
		}

		// Important: This VNCClientsTable stores all VNC Client
		private static Dictionary<int, VNCClient> s_CurVNCClientsTable = null;

		#endregion Static Members, Events & Functions

		public VNCClient()
		{
			this._vnc_handler_index = -1;
			this._bFlag_IsVNCRunning = false;
		}

		// ---------------------------------------------------------------------------------------------------

		#region Priate Functions

		/// <summary>
		/// Write debug message in VNC debug log file
		/// </summary>
		/// <param name="sMsg">debug message</param>
		private void VNCDebugLog(string sMsg)
		{
			try
			{
				WriteDebugLog(6, sMsg);
			}
			catch (Exception ex)
			{ }
		}

		#endregion Priate Functions

		// ---------------------------------------------------------------------------------------------------

		#region Public Functions of Initial/Setup/Run/Close VNC Handler

		/// <summary>
		/// Create and initial a new VNC handler for one new remote device
		/// </summary>
		/// <returns>True: Success; False: Fail</returns>
		public bool InitialVNC()
		{
			#region Step(1): Create a new VNC handler

			try
			{
				VNCDebugLog("[VNCClient] InitialVNC(): Create a new VNC handler ...\n");
				this._vnc_handler_index = CreateNewVNCHandler();
			}
			catch (Exception ex)
			{
				this._vnc_handler_index = -1;
				VNCDebugLog("[VNCClient] InitialVNC(): Create New VNCHandler Exception => " + ex.ToString() + "\n");
			}

			if (this._vnc_handler_index < 0)
			{
				VNCDebugLog("[VNCClient] InitialVNC(): Failed !!\n");
				return false;
			}

			#endregion Step(1): Create a new VNC handler

			#region Step(2): Initial VNC handler

			try
			{
				VNCDebugLog("[VNCClient] InitialVNC(): Initial the new VNC handler ...\n");

				InitialStaticMembers();

				// Add this new VNCClient in "s_CurVNCClientsTable"
				s_CurVNCClientsTable.Add(this._vnc_handler_index, this);

				// Initial success -> Set callback function
				// (1) Set the callback function that returns VNC error message
				SetVNCCallbackFun(VNCMsgCallback);

				// (2) Set the callback function that updates the progress of sending file
				SetFileTransferCallbackFun(FTProcCallback, FTDownloadProcCallback);

				this._FilesPathMapTable = new Dictionary<int, string>();

				string sDownloadFolder = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData);
				sDownloadFolder += "\\AirFusion\\Download";
				SetVNCDownloadFolderPath(sDownloadFolder);
			}
			catch (Exception ex)
			{
				VNCDebugLog("[VNCClient] InitialVNC(): Initialize New VNCHandler Exception => " + ex.ToString() + "\n");
				return false;
			}

			#endregion Step(2): Initial VNC handler

			VNCDebugLog("[VNCClient] InitialVNC(): Successfully !!\n");
			return true;
		}

		public void InitialClipboard(IntPtr hWnd)
		{
			InitVNCClipboard((byte)this._vnc_handler_index, hWnd);
		}

		public static void RemoveClipboard(IntPtr hWnd)
		{
			RemoveVNCClipboard(hWnd);
		}

		public VNC_ERR_CODE RunVNC(string sIPv4, uint uiPortNum, string sPW)
		{
			VNCConnectInfo args = new VNCConnectInfo();
			args.sIPv4 = sIPv4;
			args.uiPortNum = uiPortNum;
			args.sPassword = sPW;

			VNC_ERR_CODE RetErr = VNC_ERR_CODE.EC_NoError;

			try
			{
				// [ToDoItem] Check IP is correct or it can ping this IP
				// If IP is incorrect or Ping is failed => Return VNC_ERR_CODE.InvalidServerIP
				VNCDebugLog("[VNCClient] RunVNC(): Call VNCHandler::RunVNCHandler()...\n");

				RetErr = (VNC_ERR_CODE)RunVNCHandler((byte)this._vnc_handler_index, args);
				if (RetErr == VNC_ERR_CODE.EC_NoError)
					this._bFlag_IsVNCRunning = true;
			}
			catch (Exception exp)
			{
				VNCDebugLog("[VNCClient] RunVNC(): Exception => " + exp.ToString() + "\n");
				RetErr = VNC_ERR_CODE.EC_FailedConnect;
			}

			VNCDebugLog("[VNCClient] RunVNC(): End - Error code: " + RetErr.ToString() + "\n");
			return RetErr;
		}

		public void CloseVNC()
		{
			CloseVNCHandler(this._vnc_handler_index);
			s_CurVNCClientsTable.Remove(this._vnc_handler_index);
		}

		public void SetAirFusionScreenInfo(SCREEN_SIZE_INFO msInfo)
		{
			SetAFMiracastScreenInfo((byte)this._vnc_handler_index, (int)msInfo.uiWidth, (int)msInfo.uiHeight, msInfo.iDeltaX, msInfo.iDeltaY, msInfo.fDpiRate);
		}

		private static void InitialStaticMembers()
		{
			if (s_CurVNCClientsTable == null)
				s_CurVNCClientsTable = new Dictionary<int, VNCClient>();

			// Lock Objects
			if (s_oLock_AccessMsgQueue == null)
				s_oLock_AccessMsgQueue = new object();

			if (s_oLock_SendVncMsg == null)
				s_oLock_SendVncMsg = new object();

			if (s_oLock_AccessFTPQueue == null)
				s_oLock_AccessFTPQueue = new object();

			if (s_oLock_UpdateFTProg == null)
				s_oLock_UpdateFTProg = new object();

			// Event Queue
			if (VNCMsgEventQueue == null)
				VNCMsgEventQueue = new Queue<VNCMsgItem>();

			if (FTProgUpdateEventQueue == null)
				FTProgUpdateEventQueue = new Queue<FTUpdateItem>();

			// Callback function pointers
			if (VNCMsgCallback == null)
				VNCMsgCallback = new VNCMessageCallback(VNCMessage_CallBackFun);

			if (FTProcCallback == null)
				FTProcCallback = new FileUploadProCallback(FileUpload_CallBackFun);

			if (FTDownloadProcCallback == null)
				FTDownloadProcCallback = new FileDownloadProCallback(FileTransfer_DownloadCallBack);
		}

		#endregion Public Functions of Initial/Setup/Run/Close VNC Handler

		// ---------------------------------------------------------------------------------------------------

		#region Public Functions of Sending Input Controls

		/*
         * Windows Message Types: (參考 TightVNC code -> ClientConnection::WndProc(...))
         */

		public IntPtr WinProc(IntPtr hwnd, int msg, IntPtr wParam, IntPtr lParam, ref bool handled)
		{
			if (this._bFlag_IsVNCRunning)
			{
				try
				{
					VncWinProc((byte)this._vnc_handler_index, hwnd, msg, wParam, lParam);
				}
				catch (Exception ex)
				{
					VNCDebugLog("[VNCClient] WinProc(): msg = " + msg.ToString() + "; Exception => " + ex.ToString() + "\n");
				}
			}

			return IntPtr.Zero;
		}

		public void SendIMEString(string str)
		{
			try
			{
				SendIMEChars((byte)this._vnc_handler_index, str, str.Length);
			}
			catch (Exception ex)
			{
				VNCDebugLog("[VNCClient] SendIMEString(): str = " + str + "; Exception => " + ex.ToString() + "\n");
			}
		}

		public void SendInputVirtualKey(VIRTUAL_KEY_TYPE virKeyType)
		{
			try
			{
				// KEYEVENTF_KEYDOWN = 0x0000
				SendVirtualKey((byte)this._vnc_handler_index, (uint)virKeyType, 0x0);
			}
			catch (Exception ex)
			{
				VNCDebugLog("[VNCClient] SendInputVirtualKey(): virKeyType = " + virKeyType.ToString() + "; Exception => " + ex.ToString() + "\n");
			}
		}

		/// <summary>
		/// Sends a touch event to server
		/// </summary>
		/// <param name="type">TouchDown = 0, TouchMove = 1, TouchUp = 2</param>
		/// <param name="x_value">Touch location: X value</param>
		/// <param name="y_value">Touch location: Y value</param>
		public void SendTouchInpugControl(TouchEventType type, Int32 x_value, Int32 y_value)
		{
			try
			{
				SendTouchControl((byte)this._vnc_handler_index, (uint)type, x_value, y_value);
			}
			catch (Exception ex)
			{
				VNCDebugLog("[VNCClient] SendTouchInpugControl(): TouchEventType = " + type.ToString() + "; Exception => " + ex.ToString() + "\n");
			}
		}

		#endregion Public Functions of Sending Input Controls

		// ---------------------------------------------------------------------------------------------------

		#region The Functions that Upload Files to Server

		private Dictionary<int, string> _FilesPathMapTable;	// 針對 Upload Multi-Files 的情況，會提供一個 File ID 對應到一個 File Path

		/// <summary>
		/// Transfer the files to Server (Smart phone)
		/// </summary>
		/// <param name="strLocalFiles">Files List</param>
		private void UploadFilesToServer(List<string> strLocalFiles)
		{
			if (strLocalFiles.Count() <= 0)
			{
				// [ToDo] Error Message
				return;
			}

			// New a FTCalcProgressInfo
			_CalcProgressInfo = new FTCalcProgressInfo(strLocalFiles.Count());

			FTProgUpdateEventQueue.Clear();
			ClearFilesPathMapTable();

			// For loop to send files
			foreach (string aFilePath in strLocalFiles)
			{
				// Check if the file is existing
				if (false == System.IO.File.Exists(aFilePath))
				{
					// [ToDo] Warning Message
					_CalcProgressInfo.SendCompleteFilesCount++;
					continue;
				}

				int iFile_id = AllocateFileIndex(aFilePath);

				// Ask VNCHandler to upload the specified file to Server
				TransferFileToServer((byte)this._vnc_handler_index, aFilePath, iFile_id);
				_CalcProgressInfo.SendCompleteFilesCount++;
			}

			// Finish => Send final progress to UI
			lock (s_oLock_AccessFTPQueue)
			{
				FTProgUpdateEventQueue.Enqueue(new FTUpdateItem(this._vnc_handler_index, FileTransferType.UploadFilesFinish));
			}
			CreateMonitorProgressThread();
		}

		/// <summary>
		/// Transfer one file to Server (Smart phone)
		/// </summary>
		/// <param name="strLocalFilePath"></param>
		public void UploadOneFileToServer(string strLocalFilePath)
		{
			VNCDebugLog("[VNCClient] UploadOneFileToServer(): Upload a file to server => " + strLocalFilePath + "\n");
			if (strLocalFilePath == string.Empty)
				return;

			// New a FTCalcProgressInfo
			_CalcProgressInfo = new FTCalcProgressInfo(1);

			FTProgUpdateEventQueue.Clear();
			ClearFilesPathMapTable();

			// Check if the file is existing
			if (false == System.IO.File.Exists(strLocalFilePath))
			{
				// [ToDo] Warning Message
				VNCDebugLog("[VNCClient] UploadOneFileToServer(): File is not exist !!\n");
				return;
			}

			int iFile_id = AllocateFileIndex(strLocalFilePath);
			TransferFileToServer((byte)this._vnc_handler_index, strLocalFilePath, iFile_id);

			//// Finish => Send final progress to UI
			//lock (s_oLock_AccessQueue)
			//{
			//	FTProgUpdateEventQueue.Enqueue(new FTUpdateItem(this._vnc_handler_index, FileTransferType.UploadFilesFinish));
			//}
			//CreateMonitorProgressThread();
		}

		private int _iCurTransferFileIndex = -1;

		private int AllocateFileIndex(string strFilePath)
		{
			_iCurTransferFileIndex++;
			_FilesPathMapTable.Add(_iCurTransferFileIndex, strFilePath);

			return _iCurTransferFileIndex;
		}

		private void ClearFilesPathMapTable()
		{
			_FilesPathMapTable.Clear();
			_iCurTransferFileIndex = -1;
		}

		#endregion The Functions that Upload Files to Server

		// ---------------------------------------------------------------------------------------------------

		#region Public Functions are used to simulate user's acetion that operates smart phone (ex: Touch Sliding)

		/// <summary>
		/// Swipes view from left to right on phone screen
		/// </summary>
		public void SwipeView_LeftToRight()
		{
			VNCDebugLog("[VNCClient] SwipeView_LeftToRight(): Swipe view from left to right !!\n");
			SendTouchInpugControl(TouchEventType.SwipeLeftToRight, 0, 0);
		}

		/// <summary>
		/// Swipes view from right to left on phone screen
		/// </summary>
		public void SwipeView_RightToLeft()
		{
			VNCDebugLog("[VNCClient] SwipeView_RightToLeft(): Swipe view from right to left !!\n");
			SendTouchInpugControl(TouchEventType.SwipeRightToLeft, 0, 0);
		}

		/// <summary>
		/// Drops down notification list
		/// </summary>
		public void DropDown_Notifications()
		{
			VNCDebugLog("[VNCClient] DropDown_Notifications(): Swipe view from right to left !!\n");
			SendTouchInpugControl(TouchEventType.DropDownNotification, 0, 0);
		}

		#endregion Public Functions are used to simulate user's acetion that operates smart phone (ex: Touch Sliding)

		// ---------------------------------------------------------------------------------------------------

		#region Other public functions

		/// <summary>
		/// Modify the debug log of VNC Handler
		/// </summary>
		/// <param name="bEnable"></param>
		/// <param name="sLogFilePath"></param>
		public static void ControlVNCDebugLog(bool IsEnable, string LogFilePath, int LogLevel = 6)
		{
			if (IsEnable)
				EnableDebugLog(LogFilePath, LogLevel);
			else
				CloseDebugLog();
		}

		public bool SetVNCDownloadFolderPath(string sFolderPath)
		{
			try
			{
				if (sFolderPath == string.Empty)
					return false;

				if (sFolderPath.EndsWith("\\"))
					sFolderPath = sFolderPath.Substring(0, sFolderPath.Length - 1);

				if (!System.IO.Directory.Exists(sFolderPath))
				{
					System.IO.DirectoryInfo folderInfo = System.IO.Directory.CreateDirectory(sFolderPath);
				}

				SetDownloadFolderPath((byte)this._vnc_handler_index, sFolderPath);
			}
			catch (Exception exp)
			{
				VNCDebugLog("[VNCClient] SetVNCDownloadFolderPath(): Exception => " + exp.ToString() + "\n");
				return false;
			}

			return true;
		}

		public void RightClickToPaste()
		{
			RightClickClipboard((byte)this._vnc_handler_index);
		}

		#endregion Other public functions
	}
}