//  Copyright (C) 2003 Dennis Syrovatsky. All Rights Reserved.
//
//  This file is part of the VNC system.
//
//  The VNC system is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//  USA.
//
// TightVNC distribution homepage on the Web: http://www.tightvnc.com/
//
// If the source code for the VNC system is not available from the place
// whence you received this file, check http://www.uk.research.att.com/vnc or contact
// the authors on vnc@uk.research.att.com for information on obtaining it.

////#include "vncviewer.h"
////#include "VNCviewerApp32.h"
#include "FileTransfer.h"
#include "FileTransferItemInfo.h"

#define CLIENT_FILE_FOLDER L"C:"
#define PHONE_FILE_FOLDER L"/sdcard/"
//#define PHONE_FILE_FOLDER L"H:\\TempFolder\\"
#define PROGRESS_INTERVAL_MS  30

FileTransfer::FileTransfer(ClientConnection * pCC)
{
	m_clientconn = pCC;
	m_bUploadStarted = FALSE;
	m_bDownloadStarted = FALSE;
	m_bTransferEnable = FALSE;
	m_bReportUploadCancel = FALSE;
	m_bServerBrowseRequest = FALSE;
	m_bFirstFileDownloadMsg = TRUE;
	m_hTreeItem = NULL;

	wmemset(m_ClientFilename, 0, rfbMAX_PATH);
	wcscpy(m_ClientPath, CLIENT_FILE_FOLDER);
	wmemset(m_ServerFilename, 0, rfbMAX_PATH);
	wcscpy(m_ServerPath, PHONE_FILE_FOLDER);

	m_numOfFilesToDownload = -1;
	m_currentDownloadIndex = -1;

	FileUploadProgCallback = nullptr;
}

FileTransfer::~FileTransfer()
{
	m_FTClientItemInfo.Free();
	m_FTServerItemInfo.Free();
}

// This method will be called, each time window message 'IDC_FTCOPY' is sent for
// file download. In following cases IDC_FTCOPY will be sent for file download.
// 1. From UI, when download button is clicked
// 2. From 'FileTransferDownload' method after each file download is complete.
// This method will keep track of number of selected files for which download
// request is not sent. With each call it will send download request for one file
// and will return. After the completion of file download 'FileTransferDownload'
// method will do a PostMessage for 'IDC_FTCOPY', which will invoke this method
// again and it will send another request for file download if download request for
// some of the selected files is still pending.
BOOL
	FileTransfer:: SendMultipleFileDownloadRequests()
{
	return TRUE;
}

void
	FileTransfer::FileTransferUpload(wchar_t* wsClientFilePath, int iFileId)
{
	DWORD sz_rfbFileSize;
	DWORD sz_rfbBlockSize= 8192;
	DWORD dwNumberOfBytesRead = 0;
	unsigned int mTime = 0;
	wchar_t wpath[rfbMAX_PATH];	// File's path (wchar)
	BOOL bResult;

	// Send initial progress value
	if(FileUploadProgCallback != nullptr)
		FileUploadProgCallback(m_clientconn->m_vnc_index, iFileId, 0, FPS_Transfering);

	wcscpy(wpath, wsClientFilePath);
	// Get file name
	wmemset(m_ClientFilename, 0, rfbMAX_PATH);
	GetFileNameW(wpath, m_ClientFilename);

	WIN32_FIND_DATAW FindFileData;
	UINT savedErrorMode = SetErrorMode(SEM_FAILCRITICALERRORS);
	HANDLE hFile = FindFirstFileW(wpath, &FindFileData);
	SetErrorMode(savedErrorMode);
	if (hFile == INVALID_HANDLE_VALUE) {
		int err = GetLastError();

		vnclog.Print(1, "[ERROR] FileTransfer::FileTransferUpload(): Could not find selected file, can't upload!! (Error code = %d)\n", err);

		if(FileUploadProgCallback != nullptr)
			FileUploadProgCallback(m_clientconn->m_vnc_index, iFileId, 100, FPS_Error);

		// Send Error Message
		//this->m_clientconn->SendVncMessage(MT_ErrorMessage, FC_FileUpload, EC_OtherError, nullptr);
		this->m_clientconn->SendVncMessage(MT_ErrorMessage, FC_FileUpload, EC_CreateFileFailed, nullptr);
		return;
	}
	else if ((FindFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0) {
		vnclog.Print(1, "[ERROR] FileTransfer::FileTransferUpload(): Cannot upload a directory!!\n");

		// Continue with upload of other files.

		if(FileUploadProgCallback != nullptr)
			FileUploadProgCallback(m_clientconn->m_vnc_index, iFileId, 100, FPS_Error);

		//this->m_clientconn->SendVncMessage(MT_ErrorMessage, FC_FileUpload, EC_OtherError, nullptr);
		this->m_clientconn->SendVncMessage(MT_ErrorMessage, FC_FileUpload, EC_CreateFileFailed, nullptr);
		return;
	} else {
		sz_rfbFileSize = FindFileData.nFileSizeLow;
		mTime = FiletimeToTime70(FindFileData.ftLastWriteTime);
		wcscpy(m_ServerFilename, FindFileData.cFileName);
	}
	FindClose(hFile);

	if ((sz_rfbFileSize != 0) && (sz_rfbFileSize <= sz_rfbBlockSize))
		sz_rfbBlockSize = sz_rfbFileSize;

	vnclog.Print(1, "[VNC] FileTransfer::FileTransferUpload(): Stat to CreateFileW()!!\n");
	m_hFiletoRead = CreateFileW(wpath, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, NULL);
	if (m_hFiletoRead == INVALID_HANDLE_VALUE) {
		// [ErrorMsg] SetWindowText(m_hwndFTStatus, "Upload failed: could not open selected file");
		DWORD dwError = GetLastError();
		if (dwError == 32) { // Share violation, try different combination
			m_hFiletoRead = CreateFileW(wpath, GENERIC_READ, FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, NULL);
			if  (m_hFiletoRead == INVALID_HANDLE_VALUE && GetLastError() == 32) {
				m_hFiletoRead = CreateFileW(wpath, GENERIC_READ, FILE_SHARE_READ|FILE_SHARE_DELETE, NULL, OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, NULL);
				if  (m_hFiletoRead == INVALID_HANDLE_VALUE && GetLastError() == 32) {
					m_hFiletoRead = CreateFileW(wpath, GENERIC_READ, FILE_SHARE_READ|FILE_SHARE_WRITE|FILE_SHARE_DELETE, NULL, OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, NULL);
				}
			}
		}
		if (m_hFiletoRead == INVALID_HANDLE_VALUE) // still error after trial
		{
			vnclog.Print(1, "[ERROR] FileTransfer::FileTransferUpload(): Upload failed: could not open selected file due to reason %d!!\n", GetLastError());
			if(FileUploadProgCallback != nullptr)
				FileUploadProgCallback(m_clientconn->m_vnc_index, iFileId, 100, FPS_Error);

			// Send Error Message
			this->m_clientconn->SendVncMessage(MT_ErrorMessage, FC_FileUpload, EC_CreateFileFailed, nullptr);
			return;
		}
	}

	wsprintfW(wpath, L"%s%s", m_ServerPath, m_ClientFilename);
	////ConvertPathW(wpath);
	int pathLen = WideCharToMultiByte(CP_UTF8, 0, wpath, (int) wcslen(wpath), NULL, 0, NULL, NULL);
	char *cpath = new char[pathLen];
	WideCharToMultiByte(CP_UTF8, 0, wpath, (int) wcslen(wpath), cpath, pathLen, NULL, NULL);

	char *pAllFURMessage = new char[sz_rfbFileUploadRequestMsg + pathLen];
	rfbFileUploadRequestMsg *pFUR = (rfbFileUploadRequestMsg *) pAllFURMessage;
	char *pFollowMsg = &pAllFURMessage[sz_rfbFileUploadRequestMsg];
	pFUR->type = rfbFileUploadRequest;
	pFUR->compressedLevel = 0;
	pFUR->fNameSize = Swap16IfLE(pathLen);
	pFUR->position = Swap32IfLE(0);
	memcpy(pFollowMsg, cpath, pathLen);

	vnclog.Print(1, "[VNC] FileTransfer::FileTransferUpload(): Start send data => m_clientconn->WriteExact(...) \n");
	m_clientconn->WriteExact(pAllFURMessage, sz_rfbFileUploadRequestMsg + pathLen);
	delete [] pAllFURMessage;
	delete [] cpath;

	if (sz_rfbFileSize == 0) {
		// If user decide to send a file that size is zero (Empty file)
		SendFileUploadDataMessage(mTime);
	} else {
		int iCurrenProgress = 0;
		DWORD dwPortionRead = 0;
		char *pBuff = new char [sz_rfbBlockSize];
		m_bUploadStarted = TRUE;

		while(m_bUploadStarted) {
			if (m_bTransferEnable == FALSE) {
				// File transfer is canceled !!

				char reason[] = "File transfer canceled by user";
				int reasonLen = strlen(reason);
				char *pFUFMessage = new char[sz_rfbFileUploadFailedMsg + reasonLen];
				rfbFileUploadFailedMsg *pFUF = (rfbFileUploadFailedMsg *) pFUFMessage;
				char *pReason = &pFUFMessage[sz_rfbFileUploadFailedMsg];

				pFUF->type = rfbFileUploadFailed;
				pFUF->reasonLen = Swap16IfLE(reasonLen);
				memcpy(pReason, reason, reasonLen);
				m_clientconn->WriteExact(pFUFMessage, sz_rfbFileUploadFailedMsg + reasonLen);
				delete [] pFUFMessage;

				if(FileUploadProgCallback != nullptr)
					FileUploadProgCallback(m_clientconn->m_vnc_index, iFileId, 100, FPS_Cancel);

				return;
			}

			bResult = ReadFile(m_hFiletoRead, pBuff, sz_rfbBlockSize, &dwNumberOfBytesRead, NULL);
			if (bResult && dwNumberOfBytesRead == 0) {
				/* This is the end of the file. */
				SendFileUploadDataMessage(mTime);
				break;
			}

			SendFileUploadDataMessage((unsigned short)dwNumberOfBytesRead, pBuff);
			dwPortionRead += dwNumberOfBytesRead;
			int iProgress = ((LONGLONG)dwPortionRead * 100)/ sz_rfbFileSize;

			if((iProgress > iCurrenProgress) && (iProgress < 100))
			{
				iCurrenProgress = iProgress;
				if(FileUploadProgCallback != nullptr)
				{
					static DWORD dwTickPre = 0;
					DWORD dwTickNow = GetTickCount();
					if (dwTickNow - dwTickPre < PROGRESS_INTERVAL_MS && iCurrenProgress < 95) {
						// skip if progress comes too soon
						continue;
					}
					else {
						dwTickPre = dwTickNow;
						FileUploadProgCallback(m_clientconn->m_vnc_index, iFileId, iCurrenProgress, FPS_Transfering);
					}
				}
			}
		}

		CloseHandle(m_hFiletoRead);
	}

	m_bUploadStarted = FALSE;

	// Send Final Progress Value
	if(FileUploadProgCallback != nullptr)
		FileUploadProgCallback(m_clientconn->m_vnc_index, iFileId, 100, FPS_Complete);
}

void
	FileTransfer::CloseUndoneFileTransfers()
{
	if (m_bUploadStarted) {
		m_bUploadStarted = FALSE;
		CloseHandle(m_hFiletoRead);
	}

	if (m_bDownloadStarted) {
		m_bDownloadStarted = FALSE;
		m_bFirstFileDownloadMsg = TRUE;
		m_currentDownloadIndex = -1;
		m_numOfFilesToDownload = -1;
		CloseHandle(m_hFiletoWrite);
		DeleteFileW(m_DownloadFilename);
	}
}

void
	FileTransfer::FileTransferDownload()
{
	rfbFileDownloadDataMsg fdd;
	m_clientconn->ReadExact((char *)&fdd, sz_rfbFileDownloadDataMsg);
	fdd.compressedSize = Swap16IfLE(fdd.compressedSize);
	fdd.realSize = Swap16IfLE(fdd.realSize);
	fdd.totalSize = Swap32IfLE(fdd.totalSize);
	// Source file name length
	fdd.nameLen = Swap16IfLE(fdd.nameLen);

	if (m_bFirstFileDownloadMsg) {
		// First to receive sz_rfbFileDownloadDataMsg => Get the file path and create the file at Client
		vnclog.Print(5, "[VNC] ileTransfer::FileTransferDownload(): To download one frle from Android phone ...!\n");

		m_dwDownloadBlockSize = fdd.compressedSize;
		m_bTransferEnable = TRUE;

		// Read file name
		if(fdd.nameLen <= 0)
		{
			// Stop download after receive empty file name
			m_bDownloadStarted = FALSE;
			// [ToDo] Send error to server?
			return;
		}
		char unixName[rfbMAX_PATH] = {0};
		m_clientconn->ReadExact(unixName, fdd.nameLen);

		// Convert char* to wchar_t* (File name)
		int wNameLen = MultiByteToWideChar (CP_UTF8, 0, unixName, -1, NULL, 0);
		WCHAR* wFileName = new WCHAR[wNameLen + 1];
		wmemset(wFileName, 0, wNameLen);
		MultiByteToWideChar(CP_UTF8, 0, unixName, -1, wFileName, wNameLen);
		wmemset(m_ServerFilename, 0, rfbMAX_PATH);
		wcscpy(m_ServerFilename, wFileName);
		delete [] wFileName;

		m_dwDownloadRead = 0;
		m_wCurDownloadProgress = 0;
		m_bFirstFileDownloadMsg = FALSE;

		// Send the Event that start to download file from server to UI
		if(FileDownloadProgCallback != nullptr)
			FileDownloadProgCallback(m_clientconn->m_vnc_index, m_ServerFilename, 0, FPS_Transfering);

		// Combine the file path with file name
		wchar_t path[rfbMAX_PATH];
		wsprintfW(path, L"%s\\%s", m_ClientPath, m_ServerFilename);
		wmemset(m_DownloadFilename, 0, rfbMAX_PATH);
		wcscpy(m_DownloadFilename, path);

		vnclog.Print(5, _T("[VNC] ileTransfer::FileTransferDownload(): File Path = %ws!\n"), path);
		// Create file
		m_hFiletoWrite = CreateFileW(path, GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE,
			NULL, CREATE_ALWAYS, FILE_FLAG_SEQUENTIAL_SCAN, NULL);

		// Check the total size of downloaded file
		m_dwSizeDownloadFile = fdd.totalSize;
		if(m_dwSizeDownloadFile < 0)
		{
			// [Error]
			CancelDownload("The total size of file < 0");
			m_clientconn->SendVncMessage(MT_ErrorMessage, FC_FileDownload, EC_InvalidFileSize, nullptr);

			if(FileDownloadProgCallback != nullptr)
				FileDownloadProgCallback(m_clientconn->m_vnc_index, m_ServerFilename, 0, FPS_Cancel);

			return;
		}

		m_bDownloadStarted = TRUE;
	}
	else
	{
		char unixName[rfbMAX_PATH] = {0};
		m_clientconn->ReadExact(unixName, fdd.nameLen);
	}

	if ((fdd.realSize == 0) && (fdd.compressedSize == 0)) {
		unsigned int mTime;
		m_clientconn->ReadExact((char *) &mTime, sizeof(unsigned int));
		if (m_hFiletoWrite == INVALID_HANDLE_VALUE) {
			CancelDownload("Could not create file");
			////MessageBox(m_hwndFileTransfer, "Download failed: could not create local file", "Download Failed", MB_ICONEXCLAMATION | MB_OK);
			m_clientconn->SendVncMessage(MT_ErrorMessage, FC_FileDownload, EC_CreateFileFailed, nullptr);

			if(FileDownloadProgCallback != nullptr)
				FileDownloadProgCallback(m_clientconn->m_vnc_index, m_ServerFilename, 0, FPS_Cancel);

			return;
		}

		FILETIME Filetime;
		Time70ToFiletime(mTime, &Filetime);
		SetFileTime(m_hFiletoWrite, &Filetime, &Filetime, &Filetime);
		CloseHandle(m_hFiletoWrite);

		m_bFirstFileDownloadMsg = TRUE;
		m_bDownloadStarted = FALSE;

		// Send the final progress event
		if(FileDownloadProgCallback != nullptr)
			FileDownloadProgCallback(m_clientconn->m_vnc_index, m_ServerFilename, 100, FPS_Complete);

		return;
	}

	char * pBuff = new char [fdd.compressedSize];
	DWORD dwNumberOfBytesWritten;
	m_clientconn->ReadExact(pBuff, fdd.compressedSize);
	////ProcessDlgMessage(m_hwndFileTransfer);
	if (!m_bTransferEnable) {
		CancelDownload("Download cancelled by user");
		delete [] pBuff;
		return;
	}

	if (m_hFiletoWrite == INVALID_HANDLE_VALUE) {
		CancelDownload("Could not create file");
		m_clientconn->SendVncMessage(MT_ErrorMessage, FC_FileDownload, EC_CreateFileFailed, nullptr);

		if(FileDownloadProgCallback != nullptr)
			FileDownloadProgCallback(m_clientconn->m_vnc_index, m_ServerFilename, 0, FPS_Cancel);

		delete [] pBuff;
		return;
	}

	WriteFile(m_hFiletoWrite, pBuff, fdd.compressedSize, &dwNumberOfBytesWritten, NULL);
	m_dwDownloadRead += dwNumberOfBytesWritten;

	int iProgress = (LONGLONG)(m_dwDownloadRead * 100) / m_dwSizeDownloadFile;
	if((iProgress > m_wCurDownloadProgress) && (iProgress < 100))
	{
		m_wCurDownloadProgress = iProgress;
		if(FileDownloadProgCallback != nullptr)
		{
			static DWORD dwTickPreDown = 0;
			DWORD dwTickNow = GetTickCount();
			if (dwTickNow - dwTickPreDown < PROGRESS_INTERVAL_MS && m_wCurDownloadProgress < 95) {
				// skip if progress comes too soon
			}
			else {
				dwTickPreDown = dwTickNow;
				FileDownloadProgCallback(m_clientconn->m_vnc_index, m_ServerFilename, m_wCurDownloadProgress, FPS_Transfering);
			}
		}
	}

	delete [] pBuff;
}

void
	FileTransfer::CancelDownload(char *reason)
{
	SendFileDownloadCancelMessage(strlen(reason), reason);
	CloseUndoneFileTransfers();
	m_bDownloadStarted = FALSE;
}

void
	FileTransfer::ShowServerItems()
{
	// [TBD] If Air Fusion needs to show the folder info of server (smart phone)
}

void
	FileTransfer::SendFileListRequestMessage(char *filename, unsigned char flags)
{
	// [TBD] If Air Fusion needs to show the folder info of server (smart phone)
}

void
	FileTransfer::SendFileUploadDataMessage(unsigned int mTime)
{
	rfbFileUploadDataMsg msg;
	msg.type = rfbFileUploadData;
	msg.compressedLevel = 0;
	msg.realSize = Swap16IfLE(0);
	msg.compressedSize = Swap16IfLE(0);

	CARD32 time32 = Swap32IfLE((CARD32)mTime);

	char data[sz_rfbFileUploadDataMsg + sizeof(CARD32)];
	memcpy(data, &msg, sz_rfbFileUploadDataMsg);
	memcpy(&data[sz_rfbFileUploadDataMsg], &time32, sizeof(CARD32));

	m_clientconn->WriteExact(data, sz_rfbFileUploadDataMsg + sizeof(CARD32));
}

void
	FileTransfer::SendFileUploadDataMessage(unsigned short size, char *pFile)
{
	int msgLen = sz_rfbFileUploadDataMsg + size;
	char *pAllFUDMessage = new char[msgLen];
	rfbFileUploadDataMsg *pFUD = (rfbFileUploadDataMsg *) pAllFUDMessage;
	char *pFollow = &pAllFUDMessage[sz_rfbFileUploadDataMsg];
	pFUD->type = rfbFileUploadData;
	pFUD->compressedLevel = 0;
	pFUD->realSize = Swap16IfLE(size);
	pFUD->compressedSize = Swap16IfLE(size);
	memcpy(pFollow, pFile, size);
	m_clientconn->WriteExact(pAllFUDMessage, msgLen);
	delete [] pAllFUDMessage;
}

void
	FileTransfer::SendFileDownloadCancelMessage(unsigned short reasonLen, char *reason)
{
	int msgLen = sz_rfbFileDownloadCancelMsg + reasonLen;
	char *pAllFDCMessage = new char[msgLen];
	rfbFileDownloadCancelMsg *pFDC = (rfbFileDownloadCancelMsg *) pAllFDCMessage;
	char *pFollow = &pAllFDCMessage[sz_rfbFileDownloadCancelMsg];
	pFDC->type = rfbFileDownloadCancel;
	pFDC->reasonLen = Swap16IfLE(reasonLen);
	memcpy(pFollow, reason, reasonLen);
	m_clientconn->WriteExact(pAllFDCMessage, msgLen);
	delete [] pAllFDCMessage;
}

void
	FileTransfer::ReadUploadCancel()
{
	if (m_bUploadStarted) {
		m_bUploadStarted = FALSE;
		CloseHandle(m_hFiletoRead);
	}

	// Stop file transfer
	CloseUndoneFileTransfers();

	// Read the message
	rfbFileUploadCancelMsg msg;
	m_clientconn->ReadExact((char *)&msg, sz_rfbFileUploadCancelMsg);
	int len = Swap16IfLE(msg.reasonLen);
	char *reason = new char[len + 1];
	m_clientconn->ReadExact(reason, len);
	reason[len] = '\0';

	// Report error (only once per upload)
	if (m_bReportUploadCancel) {
		char *errmsg = new char[128 + len];
		sprintf(errmsg, "Upload failed: %s", reason);
		// [ToDo] Send the cancel reson to UI
		////MessageBox(m_hwndFileTransfer, errmsg, "Upload Failed", MB_ICONEXCLAMATION | MB_OK);
		////SetWindowText(m_hwndFTStatus, errmsg);
		vnclog.Print(1, _T("[VNC] FileTransfer::ReadUploadCancel(): Upload failed: %s\n"), reason);
		m_bReportUploadCancel = FALSE;
		delete[] errmsg;
	}

	delete[] reason;
}

void
	FileTransfer::ReadDownloadFailed()
{
	// We'll report the error only if we're actually downloading
	BOOL downloadActive = m_bDownloadStarted;

	// Stop file transfer
	CloseUndoneFileTransfers();

	// Read the message
	rfbFileDownloadFailedMsg msg;
	m_clientconn->ReadExact((char *)&msg, sz_rfbFileDownloadFailedMsg);
	int len = Swap16IfLE(msg.reasonLen);
	char *reason = new char[len + 1];
	m_clientconn->ReadExact(reason, len);
	reason[len] = '\0';

	// Report error
	if (downloadActive) {
		char *errmsg = new char[128 + len];
		sprintf(errmsg, "Download failed: %s", reason);
		// [ToDo] Send the failed reason to UI
		////MessageBox(m_hwndFileTransfer, errmsg, "Download Failed", MB_ICONEXCLAMATION | MB_OK);
		////SetWindowText(m_hwndFTStatus, errmsg);
		vnclog.Print(1, _T("[VNC] FileTransfer::ReadDownloadFailed(): Download failed: %s\n"), reason);
		delete[] errmsg;
	}

	delete[] reason;
}

unsigned int FileTransfer::FiletimeToTime70(FILETIME ftime)
{
	LARGE_INTEGER uli;
	uli.LowPart = ftime.dwLowDateTime;
	uli.HighPart = ftime.dwHighDateTime;
	uli.QuadPart = (uli.QuadPart - 116444736000000000) / 10000000;
	return uli.LowPart;
}

void FileTransfer::Time70ToFiletime(unsigned int time70, FILETIME *pftime)
{
	LONGLONG ll = Int32x32To64(time70, 10000000) + 116444736000000000;
	pftime->dwLowDateTime = (DWORD) ll;
	pftime->dwHighDateTime = (DWORD)(ll >> 32);
}

void FileTransfer::GetFileNameW(wchar_t* wsFilePath, wchar_t* wsOutFileName)
{
	wchar_t drive[_MAX_DRIVE];
	wchar_t dir[_MAX_DIR];
	wchar_t fname[_MAX_FNAME];
	wchar_t ext[_MAX_EXT];

	_wsplitpath(wsFilePath, drive, dir, fname, ext);

	wcscpy(wsOutFileName, fname);
	wcscat(wsOutFileName, ext);
}

void
	FileTransfer::ConvertPathW(wchar_t *path)
{
	int len = wcslen(path);
	if(len >= rfbMAX_PATH) return;

	if(wcscmp(path, L"") == 0)
	{
		wcscpy(path, L"/");
		return;
	}

	for(int i = (len -1); i >= 0; i--)
	{
		if(path[i] == L'\\')
			path[i] = L'/';

		path[i+1] = path[i];
	}

	path[len+1] = L'\0';
	path[0] = L'/';

	return;
}